//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
* @file     mmchs.h
* @brief    SDcard low level driver functions
*
* Copyright (C) 2012 Parrot S.A.
*
* @author   jerome.leveque@parrot.com
* @date     2012-04-01
*******************************************************************************
*/

#ifndef CYGONCE_MMCHS_H
#define CYGONCE_MMCHS_H

/*
 * Description:
 * The register offsets for the MMCHS controller registers are defined
 * below:
 */

#define MMCHS_SYSCONFIG_AUTOIDLE                (1 << 0)
#define MMCHS_SYSCONFIG_RESET                   (1 << 1)
#define MMCHS_SYSCONFIG_ENAWAKEUP               (1 << 2)
#define MMCHS_SYSCONFIG_SIDLEMODE_IGNORE        (1 << 3)
#define MMCHS_SYSCONFIG_CLOCKACTIVITY_MAINTAIN  (3 << 8)

#define MMCHS_SYSSTATUS_RESET_DONE  (1 << 0)

#define MMCHS_SYSCTL_ICE            (1 << 0)
#define MMCHS_SYSCTL_ICS            (1 << 1)
#define MMCHS_SYSCTL_CEN            (1 << 2)
#define MMCHS_SYSCTL_CLK_DIV_MAX    (0x3FF)
#define MMCHS_SYSCTL_CLK_DIV_SHIFT  (6)
#define MMCHS_SYSCTL_DTO_MAX        (0xE)
#define MMCHS_SYSCTL_DTO_MASK       (0xF)
#define MMCHS_SYSCTL_DTO_SHIFT      (16)
#define MMCHS_SYSCTL_SRA            (1 << 24)
#define MMCHS_SYSCTL_SRC            (1 << 25)
#define MMCHS_SYSCTL_SRD            (1 << 26)

#define MMCHS_CON_OD                (1 << 0)
#define MMCHS_CON_INIT              (1 << 1)
#define MMCHS_CON_HR                (1 << 2)
#define MMCHS_CON_STR               (1 << 3)
#define MMCHS_CON_MODE              (1 << 4)
#define MMCHS_CON_DW8               (1 << 5)
#define MMCHS_CON_MIT               (1 << 6)
#define MMCHS_CON_CDP               (1 << 7)
#define MMCHS_CON_WPP               (1 << 8)
#define MMCHS_CON_DVAL_33_US        (0 << 9)
#define MMCHS_CON_DVAL_231_US       (1 << 9)
#define MMCHS_CON_DVAL_1_MS         (2 << 9)
#define MMCHS_CON_DVAL_8_4_MS       (3 << 9)
#define MMCHS_CON_CTPL              (1 << 11)
#define MMCHS_CON_CEATA             (1 << 12)
#define MMCHS_CON_OBIP              (1 << 13)
#define MMCHS_CON_OBIE              (1 << 14)
#define MMCHS_CON_PADEN             (1 << 15)
#define MMCHS_CON_CLKEXTFREE        (1 << 16)

#define MMCHS_HCTL_DTW              (1 << 1)
#define MMCHS_HCTL_SDBP             (1 << 8)
#define MMCHS_HCTL_SDVS_1_8_V       (5 << 9)
#define MMCHS_HCTL_SDVS_3_0_V       (6 << 9)
#define MMCHS_HCTL_SDVS_3_3_V       (7 << 9)
#define MMCHS_HCTL_SDVS_MASK        (MMCHS_HCTL_SDVS_3_3_V)
#define MMCHS_HCTL_SBGR             (1 << 16)
#define MMCHS_HCTL_CR               (1 << 17)
#define MMCHS_HCTL_RWC              (1 << 18)
#define MMCHS_HCTL_IBG              (1 << 19)
#define MMCHS_HCTL_IWE              (1 << 24)
#define MMCHS_HCTL_INS              (1 << 25)
#define MMCHS_HCTL_REM              (1 << 26)
#define MMCHS_HCTL_OBWE             (1 << 27)

#define MMCHS_CAPA_HSS              (1 << 21)
#define MMCHS_CAPA_DS               (1 << 22)
#define MMCHS_CAPA_SRS              (1 << 23)
#define MMCHS_CAPA_VS33             (1 << 24)
#define MMCHS_CAPA_VS30             (1 << 25)
#define MMCHS_CAPA_VS18             (1 << 26)

#define MMCHS_CMD_DE                (1 << 0)
#define MMCHS_CMD_BCE               (1 << 1)
#define MMCHS_CMD_ACEN              (1 << 2)
#define MMCHS_CMD_DDIR_READ         (1 << 4)
#define MMCHS_CMD_DDIR_WRITE        (0 << 4)
#define MMCHS_CMD_MSBS              (1 << 5)
#define MMCHS_CMD_RSP_NONE          (0 << 16)
#define MMCHS_CMD_RSP_136           (1 << 16)
#define MMCHS_CMD_RSP_48            (2 << 16)
#define MMCHS_CMD_RSP_48_BUSY       (3 << 16)
#define MMCHS_CMD_RSP_MASK          (MMCHS_CMD_RSP_48_BUSY)
#define MMCHS_CMD_CCCE              (1 << 19)
#define MMCHS_CMD_CICE              (1 << 20)
#define MMCHS_CMD_DP                (1 << 21)
#define MMCHS_CMD_CMD_TYPE_OTHERS   (0 << 22)
#define MMCHS_CMD_CMD_BUS_SUSPEND   (1 << 22)
#define MMCHS_CMD_CMD_FNCT_SELECT   (2 << 22)
#define MMCHS_CMD_CMD_FNCT_ABORT    (3 << 22)
#define MMCHS_CMD_CMD_MASK          (MMCHS_CMD_CMD_FNCT_ABORT)
#define MMCHS_CMD_CMD_INDEX_SHIFT   (24)

#define MMCHS_PSTATE_CMDI           (1 << 0)
#define MMCHS_PSTATE_DATI           (1 << 1)
#define MMCHS_PSTATE_DLA            (1 << 2)
#define MMCHS_PSTATE_WTA            (1 << 8)
#define MMCHS_PSTATE_RTA            (1 << 9)
#define MMCHS_PSTATE_BWE            (1 << 10)
#define MMCHS_PSTATE_BRE            (1 << 11)
#define MMCHS_PSTATE_DLEV_0         (1 << 20)
#define MMCHS_PSTATE_DLEV_1         (1 << 21)
#define MMCHS_PSTATE_DLEV_2         (1 << 22)
#define MMCHS_PSTATE_DLEV_3         (1 << 23)
#define MMCHS_PSTATE_CLEV           (1 << 24)

#define MMCHS_BLK_BLEN_MASK         (0x7FF)
#define MMCHS_BLK_BLEN_SHIFT        (0)
#define MMCHS_BLK_NBLK_MASK         (0xFFFF)
#define MMCHS_BLK_NBLK_SHIFT        (16)

// Bit definition for:
// OMAP3630_MMCHS_STAT, OMAP3630_MMCHS_IE, OMAP3630_MMCHS_ISE
#define MMCHS_INT_CMD_COMPLETE      (1 << 0)
#define MMCHS_INT_DATA_END          (1 << 1)
#define MMCHS_INT_ERR_CMD_TIMEOUT   (1 << 16)
#define MMCHS_INT_ERR_CMD_CRC       (1 << 17)
#define MMCHS_INT_ERR_CMD_ENDBIT    (1 << 18)
#define MMCHS_INT_ERR_CMD_INDEX     (1 << 19)
#define MMCHS_INT_ERR_DATA_TIMEOUT  (1 << 20)
#define MMCHS_INT_ERR_DATA_CRC      (1 << 21)
#define MMCHS_INT_ERR_DATA_ENDBIT   (1 << 22)
#define MMCHS_INT_ERR_DMA           (1 << 29)

#define MMCHS_INT_CMD_MASK  (MMCHS_INT_CMD_COMPLETE     | \
                             MMCHS_INT_ERR_CMD_TIMEOUT  | \
                             MMCHS_INT_ERR_CMD_CRC      | \
                             MMCHS_INT_ERR_CMD_ENDBIT   | \
                             MMCHS_INT_ERR_CMD_INDEX)

#define MMCHS_INT_DATA_MASK (MMCHS_INT_DATA_END         | \
                             MMCHS_INT_ERR_DATA_TIMEOUT | \
                             MMCHS_INT_ERR_DATA_CRC     | \
                             MMCHS_INT_ERR_DATA_ENDBIT)

#define MMCHS_INT_MISC_MASK (MMCHS_INT_ERR_DMA)

#define MMCHS_INT_ALL_MASK  (MMCHS_INT_CMD_MASK         | \
                             MMCHS_INT_DATA_MASK        | \
                             MMCHS_INT_MISC_MASK)

#endif /* CYGONCE_MMCHS_H */
