//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
* @file  sdi_dma.h
* @brief DMA functions
*
* Copyright (C) 2007 Parrot S.A.
*
* @author     florent.bayendrian@parrot.fr
* @date       2006-02-27
*******************************************************************************
*/

#ifndef _SDI_DMA_H
#define _SDI_DMA_H

#include <cyg/infra/cyg_type.h>
#include "sdi.h"

#define S3C2412X_INT_DMA0 17
#define S3C2412X_INT_DMA1 18
#define S3C2412X_INT_DMA2 19
#define S3C2412X_INT_DMA3 20

#define SDI_DMA_CHAN_INT S3C2412X_INT_DMA0

#define S3C2412_DMA_BASE_ADDRESS ((cyg_uint32 *) 0x4b000000)

#define DMA_REGISTER(__N)   (* ((volatile cyg_uint32 *) (S3C2412_DMA_BASE_ADDRESS) + __N))

#define S3C2412_DMA_REG_DISRC         ((cyg_uint32) 0)
#define S3C2412_DMA_REG_DISRCC        ((cyg_uint32) 1)
#define S3C2412_DMA_REG_DIDST         ((cyg_uint32) 2)
#define S3C2412_DMA_REG_DIDSTC        ((cyg_uint32) 3)
#define S3C2412_DMA_REG_DCON          ((cyg_uint32) 4)
#define S3C2412_DMA_REG_DSTAT         ((cyg_uint32) 5)
#define S3C2412_DMA_REG_DCSRC         ((cyg_uint32) 6)
#define S3C2412_DMA_REG_DCDST         ((cyg_uint32) 7)
#define S3C2412_DMA_REG_DMASKTRIG     ((cyg_uint32) 8)
#define S3C2412_DMA_REG_DMAREQSEL     ((cyg_uint32) 9)

/*
 * Bit offsets and masks for DMA Initial Source Control Register (DISRCC)
 */
#define S3C2412_DMA_DISRCC_INC        ((cyg_uint32) (1<<0))
#define S3C2412_DMA_DISRCC_LOC        ((cyg_uint32) (1<<1))

/*
 * Bit offsets and masks for DMA Destination Control Register (DIDSTC)
 */
#define S3C2412_DMA_DIDSTC_INC        ((cyg_uint32) (1<<0))
#define S3C2412_DMA_DIDSTC_LOC        ((cyg_uint32) (1<<1))
#define S3C2412_DMA_DIDSTC_CHK_INT    ((cyg_uint32) (1<<2))

/*
 * Bit offsets and masks for DMA Control Register (DCON)
 */
#define S3C2412_DMA_DCON_RELOAD       ((cyg_uint32) (1<<22))
#define S3C2412_DMA_DCON_PADDRFIX     ((cyg_uint32) (1<<24))
#define S3C2412_DMA_DCON_SERVMODE     ((cyg_uint32) (1<<27))
#define S3C2412_DMA_DCON_TSZ          ((cyg_uint32) (1<<28))
#define S3C2412_DMA_DCON_INT          ((cyg_uint32) (1<<29))
#define S3C2412_DMA_DCON_SYNC         ((cyg_uint32) (1<<30))
#define S3C2412_DMA_DCON_DMD_HS       ((cyg_uint32) (1<<31))

#define bsS3C2412_DMA_DCON_TC         ((cyg_uint32)   0)
#define bwS3C2412_DMA_DCON_TC         ((cyg_uint32)  20)
#define bsS3C2412_DMA_DCON_DSZ        ((cyg_uint32)  20)
#define bwS3C2412_DMA_DCON_DSZ        ((cyg_uint32)   2)

/*
 * Bit offsets and masks for DMA Status Register (DSTAT)
 */
#define bsS3C2412_DMA_DSTAT_CURR_TC   ((cyg_uint32)   0)
#define bwS3C2412_DMA_DSTAT_CURR_TC   ((cyg_uint32)  20)
#define bsS3C2412_DMA_DSTAT_STAT      ((cyg_uint32)  20)
#define bwS3C2412_DMA_DSTAT_STAT      ((cyg_uint32)   2)

/*
 * Bit offsets and masks for DMA Mask Trigger Register (DMASKTRIG)
 */
#define S3C2412_DMA_DMASKTRIG_SW_TRIG ((cyg_uint32) (1<<0))
#define S3C2412_DMA_DMASKTRIG_ON      ((cyg_uint32) (1<<1))
#define S3C2412_DMA_DMASKTRIG_STOP    ((cyg_uint32) (1<<2))

/*
 * Bit offsets and masks for DMA Requeset Selection Register (DMAREQSEL)
 */
#define S3C2412_DMA_DMAREQSEL_SWHW_SEL     ((cyg_uint32) (1<<0))

#define bsS3C2412_DMA_DMAREQSEL_HWSRCSEL   ((cyg_uint32) 1)
#define bwS3C2412_DMA_DMAREQSEL_HWSRCSEL   ((cyg_uint32) 5)

#define SDI_DMA_ENABLE() \
	DMA_REGISTER(S3C2412_DMA_REG_DMASKTRIG) = S3C2412_DMA_DMASKTRIG_ON;

#define SDI_DMA_DISABLE() \
	DMA_REGISTER(S3C2412_DMA_REG_DMASKTRIG) = S3C2412_DMA_DMASKTRIG_STOP;

void sdi_dma_devconfig(cyg_uint8 * vaddr, cyg_uint32 size, cyg_bool receiving);
Cyg_ErrNo sdi_dma_finish(void);

#endif
