//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

#ifndef I2C_MASTER_H
#define I2C_MASTER_H
/*****************************************************************************
    I2C master  registers Definitions

  File: I2C_MASTER.h

    I2C_MASTER Simulations
   
                              Bruno Lacroux / may 2005 (c) Parrot SA

   List of features : 
       - 

*****************************************************************************/

////////////////////////////////////////////////////
// Parrot proprietary I2C_MASTER registers
////////////////////////////////////////////////////
#define I2CM_TRANSMIT       0x000       // I2C_MASTER Transmit Register
#define I2CM_RECEIVE        0x000       // I2C_MASTER Receive Register
#define I2CM_ITEN           0x004       // I2C_MASTER Interrupt Enable Register
#define I2CM_COMMAND        0x008       // I2C_MASTER Command Register
#define I2CM_STATUS         0x008       // I2C_MASTER Status Register
#define I2CM_PRESCALE       0x00C       // I2C_MASTER Prescale Register

// Registers bitwise definitions

// Status Register
#define I2CM_STATUS_IF      (1 << 0)  // Indicates that an interrupt has been sent
#define I2CM_STATUS_TIP     (1 << 1)  // Indicates that transfer is on going.
#define I2CM_STATUS_AL      (1 << 2)  // Indicates an arbitration lost.
#define I2CM_STATUS_BUSY    (1 << 3)  // Indicates that I2C Busy state
#define I2CM_STATUS_RXACK   (1 << 4)  // Indicates that no acknowledge was received

// Command Register
#define I2CM_COMMAND_ITACK      (1 << 0)     // Clears a pending interrupt  
#define I2CM_COMMAND_ACK        (0 << 1)    // ACK bit command
#define I2CM_COMMAND_NACK       (1 << 1)    // NACK bit command
#define I2CM_COMMAND_WR         (1 << 2)    // Write bit command
#define I2CM_COMMAND_RD         (1 << 3)    // Read bit command
#define I2CM_COMMAND_STO        (1 << 4)    // Stop bit command
#define I2CM_COMMAND_STA        (1 << 5)    // Start bit command

// Interrupt Enable register
#define I2CM_ITEN_ITEN                1  // Interrupt enabled

// Prescale register
#define I2CM_PRESCALE_100k        0x81
#define I2CM_PRESCALE_400k        0x1F

// Transmit register
#define I2CM_TRANSMIT_WRIT            0
#define I2CM_TRANSMIT_READ            1

//! write to an i2c master register
#define i2cm_write_reg( _reg_, _value_ ) \
 (*((volatile cyg_uint32 *)(PARROT5_I2CM+(_reg_))) = (cyg_uint32)(_value_))

//! read an i2c master register
#define i2cm_read_reg(_reg_ ) \
 (*((volatile cyg_uint32 *)(PARROT5_I2CM+(_reg_))))

// i2c driver error codes
enum {
    I2C_ERR_OK = 0,
    I2C_ERR_AL,
    I2C_ERR_NACK,
    I2C_ERR_TIMEOUT,
};


// public functions


#endif
