/**
 ****************************************************************************
 * Driver for Cirrus Logic CS42XX stereo audio codec.
 *
 * @file   cs42xx_supp.h
 * @brief  Supported devices and capabilities of each one.
 *
 * Code is based on the following datasheets:
 * http://www.cirrus.com/en/pubs/proDatasheet/CS4245_F1.pdf
 * http://www.cirrus.com/en/pubs/proDatasheet/CS4265_F1.pdf
 *  
 * @author David Tylski
 * @date   Mar. 2007
 * 
 * Copyright (C) 2007, Parrot S.A. - www.parrot.biz
 ****************************************************************************
 */
#ifndef _CS42XX_SUPP_H_
#define _CS42XX_SUPP_H_

#include <cyg/devs/cs42xx.h>
#include <cyg/devs/cs42xx_regs.h>



// --- Capabilities ---

// Get the address of the last register of the device memory.
extern CS42XX_REG
CS42XX_Supp_GetLastReg(CS42XX_DEVID);

// Get if this device implements an ADC.
extern cyg_bool
CS42XX_Supp_HasADC(CS42XX_DEVID);

// Get if this device implements a DAC.
extern cyg_bool
CS42XX_Supp_HasDAC(CS42XX_DEVID);

// Get if this device can handle two asynchronous master clocks for I/O.
extern cyg_bool
CS42XX_Supp_CanClockAsync(CS42XX_DEVID);

// Get if auxiliary output is present on this device.
extern cyg_bool
CS42XX_Supp_HasAuxOutput(CS42XX_DEVID);

// Get if an interrupt output pin is present on this device.
extern cyg_bool
CS42XX_Supp_HasIntrOutput(CS42XX_DEVID);

// Get the number of analog inputs on device.
extern cyg_uint8
CS42XX_Supp_GetNbAnalogInputs(CS42XX_DEVID);

// Get the number of digital inputs on device.
extern cyg_uint8
CS42XX_Supp_GetNbDigitalInputs(CS42XX_DEVID);


// --- Management of all devices ---

// Get the name of a chip, i.e. codec type and revision.
extern Cyg_ErrNo
CS42XX_Supp_GetChipName(CS42XX_DEVID, char *buf, cyg_count8 buflen);

// Initialize driver according to the specificities of this device.
extern Cyg_ErrNo
CS42XX_Supp_Init(CS42XX_DEVID);



#endif // _CS42XX_SUPP_H_

