//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 ****************************************************************************
 * Driver for Cirrus Logic CS42XX stereo audio codec.
 *
 * @file   cs42xx_hal.c
 * @brief  Hardware Abstraction Layer for CS42XX driver.
 *
 * Code is based on the following datasheets:
 * http://www.cirrus.com/en/pubs/proDatasheet/CS4245_F1.pdf
 * http://www.cirrus.com/en/pubs/proDatasheet/CS4265_F1.pdf
 * http://www.cirrus.com/en/pubs/proDatasheet/CS5345_F1.pdf
 *
 * This code has been mainly inspired from the GPIO button driver
 * (by Ivan Djelic, Parrot S.A.).
 *
 * @note  This file is specific to Parrot5(+) architecture, due to access to
 *        core registers.
 * 
 * @warning  This part of driver HAS NOT YET BEEN TESTED!
 *  
 * @author David Tylski
 * @date   Jan. 2007
 * 
 * Copyright (C) 2007, Parrot S.A. - www.parrot.biz
 ****************************************************************************
 */

#include <task_priorities.h>
#include <cyg/infra/cyg_ass.h>
#include <cyg/devs/cs42xx.h>
#include <cyg/devs/cs42xx_regs.h>
#include <cyg/devs/cs42xx_i2c.h>
#include <cyg/devs/cs42xx_evt.h>
#include <cyg/devs/cs42xx_hal.h>


// FIXME: be more platform independent!
//#include "gpio_regs.h"
// Included:
//////////////////////////////////////////////////////////////////////////////
#define _GPIO_INTC1      0x1818       // GPIO 1st IRQ configuration register
#define _GPIO_INTC2      0x181C       // GPIO 2nd IRQ configuration register

// Registers bitwise definitions

// Interrupt configuration register
#define GPIO_INTCX_POS   (0 << 6)     // Positive edge
#define GPIO_INTCX_BOTH  (1 << 6)     // Both edge
#define GPIO_INTCX_NEG   (2 << 6)     // Negative edge
#define GPIO_INTCX_LEV   (3 << 6)     // Level
#define GPIO_INTCX_LOW   (1 << 8)     // Low Level (if bit is not set Low)
#define GPIO_INTCX_CLR   (1 << 9)     // Clear
#define GPIO_INTCX_ENA   (1 << 10)    // Enable
//////////////////////////////////////////////////////////////////////////////



/*
 * Local defines.
 *****************/


/** Get the register associated to the GPIO pin.
 * This function is a bit like __GPIO() but returns the complete port and pin
 * numbers, instead of the short pin number only.
 */
#define CS42XX_HAL_GPIO_REG(_pin_)                                          \
        ( ((_pin_) >= GPIO_NB)? (_pin_):                                    \
            ((((_pin_) & 0xf) << 4) | (((_pin_) & 0x30) << 6)) )


/** Return the interrupt register associated to an interrupt vector. */
#define CS42XX_HAL_GPIOREG_INT(vector)                                      \
        ((CYGNUM_HAL_INTERRUPT_GPIO0 == (vector))?                          \
                (CYG_WORD32 *)(PARROT5_GPIO + _GPIO_INTC1):                 \
         (CYGNUM_HAL_INTERRUPT_GPIO1 == (vector))?                          \
                (CYG_WORD32 *)(PARROT5_GPIO + _GPIO_INTC2):                 \
         (NULL))

/** Mask for interrupt mode (ASIC register). */
#define CS42XX_HAL_GPIOREG_INTMODE_MASK     (GPIO_INTCX_LOW|GPIO_INTCX_LEV)


/** Priority of interrupt handler. */
#define CS42XX_HAL_INTRPRIORITY        4

/** Size of stack for the monitoring thread. */
#define CS42XX_HAL_MONITORSTACKSIZE    4096 // enough for callbacks




/*
 * Variables.
 *************/


/** Whole hardware configuration. */
static CS42XX_HALCFG CS42XXHALCfg;

#ifdef CYGPKG_DEVICES_CS42XX_USEINTR

/** Flag for the monitoring task to sleep, waiting for events, and be
 *  awakened by interrupt.
 */
static cyg_flag_t CS42XXHALFlag;

/** Callback used when interrupt occurs (non-priviledged execution). */
static CS42XX_HAL_PROCESSINTR_CB CS42XXHALCback;

/** Thread to monitor interrupts. */
static cyg_thread    CS42XXHALMonitorThread;

/** Stack for the monitoring thread. */
static char          CS42XXHALMonitorStack[CS42XX_HAL_MONITORSTACKSIZE];

/** Driver interrupt. */
static cyg_interrupt CS42XXHALInterrupt;

#endif // CYGPKG_DEVICES_CS42XX_USEINTR



/*
 * Definition of functions.
 ***************************/



#ifdef CYGPKG_DEVICES_CS42XX_USEINTR

/** Monitoring thread, requesting a callback for occurring events.
 *
 * @param data   Not used
 ****************************************************************************/
static void
CS42XX_HAL_Monitor(cyg_addrword_t data)
{
	cyg_flag_init(&CS42XXHALFlag);

	while(1)
	{
		// Wait for an interrupt to occur
		cyg_flag_wait(&CS42XXHALFlag, 1,
				CYG_FLAG_WAITMODE_OR|CYG_FLAG_WAITMODE_CLR);

		// Call the process function (if one is registered)
		if (NULL != CS42XXHALCback)
		{
			(CS42XXHALCback)();
		}
		// else no callback defined: ignore.
	}
}


/** Handle interrupt caught by ISR.
 *
 * @param vector  Interrupt that caused DSR to run
 * @param count   Number of interrupts that required this DSR
 * @param data    Registered data associated to this interrupt (not used)
 ****************************************************************************/
static void
CS42XX_HAL_Dsr(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
	// 'volatile' is required to avoid optimizations around intr. clearing
	volatile CYG_WORD32 *reg;
	CYG_WORD32 val;

	reg = CS42XX_HAL_GPIOREG_INT(vector);
	CYG_ASSERT(reg != NULL, "DSR: reg==NULL");

	// Clear interrupt and re-enable
	HAL_READ_UINT32 (reg, val);
	HAL_WRITE_UINT32(reg, val |  GPIO_INTCX_CLR);
	HAL_WRITE_UINT32(reg, val & ~GPIO_INTCX_CLR);
	
	// Wake-up monitor thread
	cyg_flag_setbits(&CS42XXHALFlag, 1);

	// Enable again interrupt requests from GPIO
	cyg_drv_interrupt_unmask(vector);
}


/** Interrupt Service Routine.
 * Nothing is done here except masking and acknowledging the interrupt, then
 * asking for the DSR execution.
 *
 * @param vector  Interrupt that caused ISR to run
 * @param data    Registered data associated to this interrupt (not used)
 *
 * @retval CYG_ISR_CALL_DSR   Request to call DSR
 ****************************************************************************/
static cyg_uint32
CS42XX_HAL_Isr(cyg_vector_t vector, cyg_addrword_t data)
{
    // Defer processing to DSR
    cyg_drv_interrupt_mask(vector);
    cyg_drv_interrupt_acknowledge(vector);
    return CYG_ISR_CALL_DSR;  // request DSR
}

#endif // CYGPKG_DEVICES_CS42XX_USEINTR


/** Reset all devices connected to the reset pin.
 *
 * @warning  This function is blocking during the whole reset application.
 * @warning  In debug mode, the function contains an assertion in case of
 *           invalid GPIO... (not really clean)
 *
 * @retval -ENOERR   Success
 * @retval -ENXIO    Bad configuration of driver
 ****************************************************************************/
Cyg_ErrNo
CS42XX_HAL_Reset(void)
{
	Cyg_ErrNo err = -ENXIO;
	cyg_uint8 gpionb;

	gpionb = __GPIO(CS42XXHALCfg.reset.pin);
	if (gpionb < GPIO_NB)
	{
		SELECT_PIN(CS42XXHALCfg.reset.pin);
		gpio_config(CS42XXHALCfg.reset.pin, GPIO_OUTPUT);
		GPIO_OUTPUT_CLEAR(gpionb);
		CS42XX_DELAY_US(50);
		
		GPIO_OUTPUT_SET(gpionb);
		CS42XX_DELAY_US(50);

		err = ENOERR;
	}
	// else HAL is not properly configured.

	return err;
}


#ifdef CYGPKG_DEVICES_CS42XX_USEINTR

/** Install the interrupt handler according to configuration previously set.
 *
 * This function shall be called only *after* the global initialization of
 * driver, since it deals with the content of audio codec registers.
 *
 * @param intrcback  Function called back when an interrupt has been caught;
 *                   can be NULL if interrupt catching is not required
 *
 * @retval  ENOERR      Success
 * @retval -ENXIO       Bad configuration of driver (process canceled)
 * @retval -EDEVNOTSUP  Device is unable to trigger events
 ****************************************************************************/
Cyg_ErrNo
CS42XX_HAL_InstallIntr(CS42XX_HAL_PROCESSINTR_CB intrcback)
{
	Cyg_ErrNo     err = -ENXIO;
	cyg_handle_t  intrhandle, monitorhandle;
	CYG_WORD     *asicreg;
	GPIO_PORT_PIN intrpin;
	cyg_vector_t  intrvector;
	cyg_uint32    intrmode;

	intrpin    =  CS42XXHALCfg.intr.pin;
	intrvector =  CS42XXHALCfg.intr.vector;
	// Driver is edge-sensitive; the positive or negative edge is determined
	// by the active state (high or low).
	intrmode   = (CS42XXHALCfg.intr.activehigh?
						GPIO_INTCX_POS: GPIO_INTCX_NEG);

	asicreg = CS42XX_HAL_GPIOREG_INT(intrvector);
	if ((NULL != asicreg) && (__GPIO(intrpin) < GPIO_NB))
	{
		// Store the process callback (can be NULL if no process)
		CS42XXHALCback = intrcback;

		// Create the appropriate interrupt handler (ISR and DSR)
		cyg_drv_interrupt_create(intrvector, CS42XX_HAL_INTRPRIORITY,
				(cyg_addrword_t)0,
				CS42XX_HAL_Isr, CS42XX_HAL_Dsr,
				&intrhandle, &CS42XXHALInterrupt);
		
		// Configure the GPIO to be the interrupt trigger on given vector
		// and activate the interrupt
		HAL_WRITE_UINT32(asicreg,
				(intrpin | intrmode | GPIO_INTCX_CLR | GPIO_INTCX_ENA));
		gpio_config(intrpin, GPIO_INPUT);
		cyg_drv_interrupt_attach(intrhandle);
		cyg_drv_interrupt_unmask(intrvector);

		// Launch monitoring thread
		cyg_thread_create(PRIO_ECOS_CS42XX_MONITOR, CS42XX_HAL_Monitor,
				(cyg_addrword_t)0, "CS42XX_event_monitor",
				(void *)CS42XXHALMonitorStack, CS42XX_HAL_MONITORSTACKSIZE,
				&monitorhandle, &CS42XXHALMonitorThread);
		cyg_thread_resume(monitorhandle);

		err = ENOERR;
	}
	// else HAL is not properly configured.

	return err;
}

#endif // CYGPKG_DEVICES_CS42XX_USEINTR


/** Set the HAL configuration on the device side.
 *
 * @pre  The driver shall be globally initialized and device ready to use
 *       (i.e. properly attached to bus, and global configuration applied).
 * @pre  No interrupt shall be active *FOR ALL DEVICES* connected to the
 *       same interrupt pin! unless there is a risk of short-circuit...
 * @pre  The interrupt vector associated to the interrupt GPIO should be
 *       masked, so that no interrupt occurs during state changing of
 *       interrupt line.
 *
 * @param devid   Symbolic device Id
 *
 * @retval  ENOERR   Configuration successfully applied on device
 * @retval -ENODEV   No such device
 ****************************************************************************/
static Cyg_ErrNo
CS42XX_HAL_SetDeviceConfig(CS42XX_DEVID devid)
{
	Cyg_ErrNo err;
	CS42XX_TRIGMODE mode = CS42XXHALCfg.intr.trigmode;

	// Set the active-high or active-low state of interrupt pin, on the
	// audio codec side.
	err = CS42XX_ConfigInt_OutputPin(devid, CS42XXHALCfg.intr.activehigh,
			CS42XX_CACHE_NOSYNC);

	// Set how the device shall trigger interrupt when an event occurs.
	// (This configuration is the same for all kinds of events.)
	if (ENOERR == err)
	{
		err = CS42XX_ConfigInt_TrigMode(devid, mode, CS42XX_CACHE_SYNC);
	}
	// else an error occurred during the configuration of output pin.

	return err;
}


/** Configure the hardware related part of audio codec.
 *
 * @retval  ENOERR   Configuration successfully applied on devices
 ****************************************************************************/
Cyg_ErrNo
CS42XX_HAL_SetAllDevicesConfig(void)
{
	Cyg_ErrNo    err;
	CS42XX_DEVID devid;
	cyg_bool     devpresent[CS42XX_NBDEVICES];
	CYG_BYTE     intmask[CS42XX_NBDEVICES];

	// For all devices, save interrupt mask and disable interrupts *now*.
	for(devid = 0; (devid < CS42XX_NBDEVICES); devid++)
	{
		intmask[devid] = CS42XX_GetIntMaskVal(devid, &err);
		if (ENOERR == err)
		{
			err = CS42XX_ConfigInt_MaskVal(devid, 0x00, CS42XX_CACHE_SYNC);
		}
		// else interrupt mask has not properly been read.

		if (-ENODEV == err)
		{
			// Device absent, but this is not critical
			devpresent[devid] = false;
		}
		// only consider the positive case of if().
	}

	// Configure interrupt registers on device (especially interrupt
	// output pin, when using multiple devices)
	if (ENOERR == err)
	{
		for(devid = 0;
		    (ENOERR == err) && (devid < CS42XX_NBDEVICES);
		    devid++)
		{
			err = CS42XX_HAL_SetDeviceConfig(devid);
		}
	}
	// else an error occurred previously.

	// Restore interrupt mask for all devices.
	if (ENOERR == err)
	{
		for(devid = 0;
		    (ENOERR == err) && (devid < CS42XX_NBDEVICES);
		    devid++)
		{
			err = CS42XX_ConfigInt_MaskVal(devid, intmask[devid],
						CS42XX_CACHE_SYNC);
		}
	}
	// else an error occurred previously.

	return err;
}


/** Get the configuration of the whole set of devices.
 *
 * Pins are returned as GPIOxx i.e. with register address and pin number
 * in the same integer.
 *
 * @param halcfg     Area to store hardware configuration
 *
 * @retval  ENOERR   Structure successfully filled
 * @retval -EINVAL   Invalid parameter
 ****************************************************************************/
Cyg_ErrNo
CS42XX_HAL_GetConfig(CS42XX_HALCFG *halcfg)
{
	Cyg_ErrNo err = -EINVAL;

	if (NULL != halcfg)
	{
		// Reset pin
		halcfg->reset.pin = CS42XXHALCfg.reset.pin;
		
		// Configuration of hardware interrupt
		halcfg->intr.pin        = CS42XXHALCfg.intr.pin;
		halcfg->intr.vector     = CS42XXHALCfg.intr.vector;
		halcfg->intr.activehigh = CS42XXHALCfg.intr.activehigh;

		// Configuration of triggering mode, from the audio
		// codec side. *May not be what is really in registers* if device
		// registers have not been synchronized.
		halcfg->intr.trigmode = CS42XXHALCfg.intr.trigmode;

		err = ENOERR;
	}
	// else structure pointer is invalid.

	return err;
}


/** Set the configuration for all devices.
 * This function should logically be called only once when initializing
 * hardware.
 *
 * Pins may be given as GPIOxx, but can also be given in short format
 * (i.e. without the address of port register).
 *
 * @param halcfg     Hardware configuration
 *
 * @retval  ENOERR   Structure successfully filled
 * @retval -EINVAL   Invalid parameters
 ****************************************************************************/
Cyg_ErrNo
CS42XX_HAL_SetConfig(CS42XX_HALCFG *halcfg)
{
	Cyg_ErrNo err = -EINVAL;
	cyg_uint8 resetpin, intrpin;

	if (NULL != halcfg)
	{
		// Get the pin number (without register address) only to check for
		// the validity of I/O (shall be strictly less than GPIO_NB).
		resetpin = __GPIO(halcfg->reset.pin);
		intrpin  = __GPIO(halcfg->intr.pin );

		if (resetpin < GPIO_NB)
		{
			// Store the reset pin.
			// Translate pin to store also the register address.
			CS42XXHALCfg.reset.pin = CS42XX_HAL_GPIO_REG(halcfg->reset.pin);

			// Get data about interrupt, if available.
			if ((NULL != CS42XX_HAL_GPIOREG_INT(halcfg->intr.vector)) &&
				(intrpin < GPIO_NB))
			{
				// All parameters OK.

				// Store the configuration of hardware interrupt.
				// Translate pin to store also the register address.
				CS42XXHALCfg.intr.pin        = CS42XX_HAL_GPIO_REG(halcfg->intr.pin);
				CS42XXHALCfg.intr.vector     = halcfg->intr.vector;
				CS42XXHALCfg.intr.activehigh = halcfg->intr.activehigh;

				// Store the configuration of triggering mode, for the audio
				// codec side. It will be set on device on the next event
				// callback registration.
				CS42XXHALCfg.intr.trigmode = halcfg->intr.trigmode;

				err = ENOERR;
			}
			else
			{
				// Fields dealing with interrupt are invalid: do not
				// consider this is an error, in case of no management of
				// device interrupts (especially if device does not support
				// interrupts!).
				err = ENOERR;
			}
		}
		// else the 'reset' field is invalid (and it is essential).
	}
	// else structure pointer is invalid.

	return err;
}

