/**
 ***************************************************************************
 * @file sta3xx.h
 *
 * @brief STA3XX digital amplifier driver.
 *
 * Copyright (C) 2007 Parrot S.A.
 *
 * @author	Frederic CHANAL
 * @date	05/2007
 ***************************************************************************
 */

#ifndef _STA3XX_
#define _STA3XX_

#include <cyg/io/io.h>
#include <cyg/io/i2c.h>
#include <pkgconf/devices_sta3xx.h>

// Registers
//////////////


/** STA3XX i2c independent devices
 */
typedef enum STA3XX_DEVICE_TAG
{
#ifdef CYGPKG_DEVICE_STA3XX_SETTINGS_STA335BW
	STA3XX_STA335BW_0,
	STA3XX_STA335BW_1,
#elif defined(CYGPKG_DEVICE_STA3XX_SETTINGS_STA326)
	STA3XX_STA326,
#endif
	STA3XX_DEVICE_COUNT,
} STA3XX_DEVICE;

/** STA3XX registers
 */
enum STA3XX_{
	//Configuration
	STA3XX_REG_ConfA	= 0x00,	
	STA3XX_REG_ConfB	= 0x01,	
	STA3XX_REG_ConfC	= 0x02,	
	STA3XX_REG_ConfD	= 0x03,	
	STA3XX_REG_ConfE	= 0x04,	
	STA3XX_REG_ConfF	= 0x05,	

	//Volume
	STA3XX_REG_Mute			= 0x06,	
	STA3XX_REG_MasterVolume	= 0x07,	
	STA3XX_REG_Chan1Volume	= 0x08,	
	STA3XX_REG_Chan2Volume	= 0x09,	
	STA3XX_REG_Chan3Volume	= 0x0A,	

	//AutoMode
	STA3XX_REG_Auto1	= 0x0B,	
	STA3XX_REG_Auto2	= 0x0C,	
	STA3XX_REG_Auto3	= 0x0D,	

	//Channel control
	STA3XX_REG_Chan1Cfg	= 0x0E,	
	STA3XX_REG_Chan2Cfg	= 0x0F,	
	STA3XX_REG_Chan3Cfg	= 0x10,	
	STA3XX_REG_Tone		= 0x11,

	//Dynamics control
	STA3XX_REG_L1ar		= 0x12,	
	STA3XX_REG_L1atrt	= 0x13, 
	STA3XX_REG_L2ar		= 0x14,	
	STA3XX_REG_L2atrt	= 0x15, 

	//User Programmable Processing
	STA3XX_REG_Cfaddr	= 0x16,
	STA3XX_REG_B1cf1	= 0x17,
	STA3XX_REG_B1cf2	= 0x18,
	STA3XX_REG_B1cf3	= 0x19,
	STA3XX_REG_B2cf1	= 0x1A,
	STA3XX_REG_B2cf2	= 0x1B,
	STA3XX_REG_B2cf3	= 0x1C,
	STA3XX_REG_A1cf1	= 0x1D,
	STA3XX_REG_A1cf2	= 0x1E,
	STA3XX_REG_A1cf3	= 0x1F,
	STA3XX_REG_A2cf1	= 0x20,
	STA3XX_REG_A2cf2	= 0x21,
	STA3XX_REG_A2cf3	= 0x22,
	STA3XX_REG_B0cf1	= 0x23,
	STA3XX_REG_B0cf2	= 0x24,
	STA3XX_REG_B0cf3	= 0x25,
	STA3XX_REG_Cfud		= 0x26,

	//Variable Max Power Correction
	STA3XX_REG_MPCC1	= 0x27,
	STA3XX_REG_MPCC2	= 0x28,

	// Variable Distortion Compensation
	STA3XX_REG_DCC1	= 0x29,
	STA3XX_REG_DCC2	= 0x2A,

	//Fault Detection Recovery
	STA3XX_REG_FDRC1	= 0x2B,
	STA3XX_REG_FDRC2	= 0x2C,

	//Status
	STA3XX_REG_Status	= 0x2D, 
};

// Configuration
//////////////////

//ConfA
#define STA3XX_ConfA_MCS_OFFSET			0
#define STA3XX_ConfA_IR_OFFSET			3
#define STA3XX_ConfA_TWRB_OFFSET		5
#define STA3XX_ConfA_TWAB_OFFSET		6
#define STA3XX_ConfA_FDRB_OFFSET		7

#define STA3XX_ConfA_MCS_768FS			(0 << STA3XX_ConfA_MCS_OFFSET)
#define STA3XX_ConfA_MCS_512FS			(1 << STA3XX_ConfA_MCS_OFFSET)
#define STA3XX_ConfA_MCS_384FS			(2 << STA3XX_ConfA_MCS_OFFSET)
#define STA3XX_ConfA_MCS_256FS			(3 << STA3XX_ConfA_MCS_OFFSET)
#define STA3XX_ConfA_MCS_128FS			(4 << STA3XX_ConfA_MCS_OFFSET)
#define STA3XX_ConfA_MCS_576FS			(5 << STA3XX_ConfA_MCS_OFFSET)

#define STA3XX_ConfA_IR_2XOVERSAMPLING	(0 << STA3XX_ConfA_IR_OFFSET)
#define STA3XX_ConfA_IR_PASSTHROUGH		(1 << STA3XX_ConfA_IR_OFFSET)
#define STA3XX_ConfA_IR_DOWNSAMPLINGBY2	(2 << STA3XX_ConfA_IR_OFFSET)

#define STA3XX_ConfA_THERMALWARNINGRECOV_ENABLE		(0 << STA3XX_ConfA_TWRB_OFFSET)
#define STA3XX_ConfA_THERMALWARNINGRECOV_DISABLE	(1 << STA3XX_ConfA_TWRB_OFFSET)

#define STA3XX_ConfA_THERMALWARNINGADJUST_ENABLE	(0 << STA3XX_ConfA_TWAB_OFFSET)
#define STA3XX_ConfA_THERMALWARNINGADJUST_DISABLE	(1 << STA3XX_ConfA_TWAB_OFFSET)

#define STA3XX_ConfA_FAULTDETECTRECOV_ENABLE		(0 << STA3XX_ConfA_FDRB_OFFSET)
#define STA3XX_ConfA_FAULTDETECTRECOV_DISABLE		(1 << STA3XX_ConfA_FDRB_OFFSET)

//ConfB
#define STA3XX_ConfB_SAI_OFFSET			0	/*SAI + SAIFB*/
#define STA3XX_ConfB_DSCKE_OFFSET		5
#define STA3XX_ConfB_C1IM_OFFSET		6
#define STA3XX_ConfB_C2IM_OFFSET		7

#define STA3XX_ConfB_32FS_I2S_15BIT					0x0c		/**< I2S 15bit */
#define STA3XX_ConfB_32FS_LRJUST_16BIT				0x0e		/**< Left/Right-Justified 16bit */
/*Bug spec ??*/#define STA3XX_ConfB_48FS_I2S_24BIT	0x00		/**< I2S 24bit */
#define STA3XX_ConfB_48FS_I2S_20BIT					0x04		/**< I2S 20bit */
#define STA3XX_ConfB_48FS_I2S_18BIT					0x08		/**< I2S 18bit */
/*Bug spec ??*/#define STA3XX_ConfB_48FS_MSBI2S_16BIT	0x0c		/**< MSB first I2S 16bit */
#define STA3XX_ConfB_48FS_LSBI2S_16BIT				0x1c		/**< LSB first I2S 16bit */
#define STA3XX_ConfB_48FS_LEFTJUST_24BIT			0x01		/**< Left-justified 24bit */
#define STA3XX_ConfB_48FS_LEFTJUST_20BIT			0x05		/**< Left-justified 20bit */
#define STA3XX_ConfB_48FS_LEFTJUST_18BIT			0x09		/**< Left-justified 18bit */
#define STA3XX_ConfB_48FS_LEFTJUST_16BIT			0x0d		/**< Left-justified 16bit */
#define STA3XX_ConfB_48FS_RIGHTJUST_24BIT			0x02		/**< Right-justified 24bit */
#define STA3XX_ConfB_48FS_RIGHTJUST_20BIT			0x06		/**< Right-justified 20bit */
#define STA3XX_ConfB_48FS_RIGHTJUST_18BIT			0x0a		/**< Right-justified 18bit */
#define STA3XX_ConfB_48FS_RIGHTJUST_16BIT			0x0e		/**< Right-justified 16bit */
#define STA3XX_ConfB_64FS_I2S_24BIT					0x00		/**< I2S 24bit */
#define STA3XX_ConfB_64FS_I2S_20BIT					0x04		/**< I2S 20bit */
#define STA3XX_ConfB_64FS_I2S_18BIT					0x08		/**< I2S 18bit */
#define STA3XX_ConfB_64FS_MSBI2S_16BIT				0x00		/**< MSB first I2S 16bit */
#define STA3XX_ConfB_64FS_LSBI2S_16BIT				0x1c		/**< LSB first I2S 16bit */
#define STA3XX_ConfB_64FS_LEFTJUST_24BIT			0x01		/**< Left-justified 24bit */
#define STA3XX_ConfB_64FS_LEFTJUST_20BIT			0x05		/**< Left-justified 20bit */
#define STA3XX_ConfB_64FS_LEFTJUST_18BIT			0x09		/**< Left-justified 18bit */
#define STA3XX_ConfB_64FS_LEFTJUST_16BIT			0x0d		/**< Left-justified 16bit */
#define STA3XX_ConfB_64FS_RIGHTJUST_24BIT			0x02		/**< Right-justified 24bit */
#define STA3XX_ConfB_64FS_RIGHTJUST_20BIT			0x06		/**< Right-justified 20bit */
#define STA3XX_ConfB_64FS_RIGHTJUST_18BIT			0x0a		/**< Right-justified 18bit */
#define STA3XX_ConfB_64FS_RIGHTJUST_16BIT			0x0e		/**< Right-justified 16bit */

#define STA3XX_ConfB_DELAYSERIALCLOCK_ENABLE	(1 << STA3XX_ConfB_DSCKE_OFFSET)
#define STA3XX_ConfB_DELAYSERIALCLOCK_DISABLE	(0 << STA3XX_ConfB_DSCKE_OFFSET)

#define STA3XX_ConfB_MAP_CHAN1_LEFT			(0 << STA3XX_ConfB_C1IM_OFFSET)
#define STA3XX_ConfB_MAP_CHAN1_RIGHT		(1 << STA3XX_ConfB_C1IM_OFFSET)
#define STA3XX_ConfB_MAP_CHAN2_LEFT			(0 << STA3XX_ConfB_C2IM_OFFSET)
#define STA3XX_ConfB_MAP_CHAN2_RIGHT		(1 << STA3XX_ConfB_C2IM_OFFSET)

//ConfC
#define STA3XX_ConfC_OM_OFFSET			0
#define STA3XX_ConfC_CSZ_OFFSET			2
#define STA3XX_ConfC_OCRB_OFFSET		7

#define STA3XX_ConfC_OUTPUT_STA50X				(2 << STA3XX_ConfC_OM_OFFSET)	/**< STA500/505/508 */
#define STA3XX_ConfC_OUTPUT_VARIABLE			(3 << STA3XX_ConfC_OM_OFFSET)	/**< Variable Compensation */
#define STA3XX_ConfC_OVERCURRENTWARNING_ENABLE	(0 << STA3XX_ConfC_OCRB_OFFSET)			/**< Over-current warning enable */
#define STA3XX_ConfC_OVERCURRENTWARNING_DISABLE	(1 << STA3XX_ConfC_OCRB_OFFSET)			/**< Over-current warning disable */

#ifdef CYGPKG_DEVICE_STA3XX_SETTINGS_STA335BW
	#define STA3XX_ConfC_COMPENSATINGPULSE(x)		((x&0x0f) << STA3XX_ConfC_CSZ_OFFSET)	/**< Compensating Pulse Size for STA33x. 0 to 15 clock period. */
#elif defined(CYGPKG_DEVICE_STA3XX_SETTINGS_STA326)
	#define STA3XX_ConfC_COMPENSATINGPULSE(x)		((x&0x1f) << STA3XX_ConfC_CSZ_OFFSET)	/**< Compensating Pulse Size for STA32x. 0 to 31 clock period. */
#endif

//ConfD
#define STA3XX_ConfD_HPB_OFFSET			0
#define STA3XX_ConfD_DEMP_OFFSET		1
#define STA3XX_ConfD_DSPB_OFFSET		2
#define STA3XX_ConfD_PSL_OFFSET			3
#define STA3XX_ConfD_BQL_OFFSET			4
#define STA3XX_ConfD_DRC_OFFSET			5
#define STA3XX_ConfD_ZDE_OFFSET			6
#define STA3XX_ConfD_MME_OFFSET			7

#define STA3XX_ConfD_ACHIGHPASS_ENABLE	(0 << STA3XX_ConfD_HPB_OFFSET)	///< AC Coupling High-Pass filter enabled
#define STA3XX_ConfD_ACHIGHPASS_DISABLE	(1 << STA3XX_ConfD_HPB_OFFSET)	///< AC Coupling High-Pass filter disabled

#define STA3XX_ConfD_NODEEMPHASIS		(0 << STA3XX_ConfD_DEMP_OFFSET)	///< No De-emphasis
#define STA3XX_ConfD_DEEMPHASIS			(1 << STA3XX_ConfD_DEMP_OFFSET)	///< De-emphasis

#define STA3XX_ConfD_DSP_NORMAL			(0 << STA3XX_ConfD_DSPB_OFFSET)	///< Normal operation
#define STA3XX_ConfD_DSP_BYPASS			(1 << STA3XX_ConfD_DSPB_OFFSET)	///< Bypass of EQ and Mixing Functionality

#define STA3XX_ConfD_POSTSCALE_CHANNEL	(0 << STA3XX_ConfD_PSL_OFFSET)	///< Each channel uses individual Post-Scale value
#define STA3XX_ConfD_POSTSCALE_UNIQUE	(1 << STA3XX_ConfD_PSL_OFFSET)	///< Each channel uses Channel 1 Post-Scale value

#define STA3XX_ConfD_BIQUAD_CHANNEL		(0 << STA3XX_ConfD_BQL_OFFSET)	///< Each channel uses coefficient value
#define STA3XX_ConfD_BIQUAD_UNIQUE		(1 << STA3XX_ConfD_BQL_OFFSET)	///< Each channel uses Channel 1 coefficient values

#define STA3XX_ConfD_ANTICLIPPING		(0 << STA3XX_ConfD_DRC_OFFSET)	///< Limiters act as Anti-Clipping mode
#define STA3XX_ConfD_DYNAMICRANGECOMP	(1 << STA3XX_ConfD_DRC_OFFSET)	///< Limiters act as Dynamic Range Compression mode

#define STA3XX_ConfD_ZERODETECTMUTE_ENABLE	(1 << STA3XX_ConfD_ZDE_OFFSET)	///< Enables automatic zero-detect mute
#define STA3XX_ConfD_ZERODETECTMUTE_DISABLE	(0 << STA3XX_ConfD_ZDE_OFFSET)	///< Enables automatic zero-detect mute

#define STA3XX_ConfD_SUBMIX_DISABLE		(0 << STA3XX_ConfD_MME_OFFSET)	///< Sub-mix into Left/Right disabled
#define STA3XX_ConfD_SUBMIX_ENABLE		(1 << STA3XX_ConfD_MME_OFFSET)	///< Sub-mix into Left/Right enabled

//ConfE
#define STA3XX_ConfE_MPCV_OFFSET		0	/*MPCV+MPC*/
#define STA3XX_ConfE_NSBW_OFFSET		2
#define STA3XX_ConfE_AME_OFFSET			3
#define STA3XX_ConfE_PWMS_OFFSET		4
#define STA3XX_ConfE_DCCV_OFFSET		5
#define STA3XX_ConfE_ZCE_OFFSET			6
#define STA3XX_ConfE_SVE_OFFSET			7

#define STA3XX_ConfE_MAXPOWERCORRECTION_DISABLE		(0 << STA3XX_ConfE_MPCV_OFFSET)	///< Disable MPC
#define STA3XX_ConfE_MAXPOWERCORRECTION_STANDARD	(2 << STA3XX_ConfE_MPCV_OFFSET)	///< Use MPC with standard coefficient
#define STA3XX_ConfE_MAXPOWERCORRECTION_COEFF		(3 << STA3XX_ConfE_MPCV_OFFSET)	///< Use MPC with coefficient from MPCC bits

#define STA3XX_ConfE_NOISESHAPER_4THORDER			(0 << STA3XX_ConfE_NSBW_OFFSET)	///< 4th order noise shaper
#define STA3XX_ConfE_NOISESHAPER_3RDORDER			(1 << STA3XX_ConfE_NSBW_OFFSET)	///< 3rd order noise shaper

#define STA3XX_ConfE_AMMODE_DISABLE				(0 << STA3XX_ConfE_AME_OFFSET)	///< Normal DDX operation
#define STA3XX_ConfE_AMMODE_ENABLE				(1 << STA3XX_ConfE_AME_OFFSET)	///< AM reduction mode DDX operation

#define STA3XX_ConfE_PWM_NORMAL					(0 << STA3XX_ConfE_PWMS_OFFSET)	///< Normal PWM Speed (384kHz)
#define STA3XX_ConfE_PWM_ODD					(1 << STA3XX_ConfE_PWMS_OFFSET)	///< Odd PWM Speed (341.3kHz)

#define STA3XX_ConfE_DCCV_PRESET				(0 << STA3XX_ConfE_DCCV_OFFSET)	///< Uses preset DC coefficients.
#define STA3XX_ConfE_DCCV_DCCCOEF				(1 << STA3XX_ConfE_DCCV_OFFSET)	///< Uses DCC coefficients.

#define STA3XX_ConfE_ZEROCROSSING_ENABLE		(1 << STA3XX_ConfE_ZCE_OFFSET)	///< Volume adjustments will only occur at digital zero-crossing
#define STA3XX_ConfE_ZEROCROSSING_DISABLE		(0 << STA3XX_ConfE_ZCE_OFFSET)	///< Volume adjustments will occur immediatly

#define STA3XX_ConfE_SOFTVOLUMEUPDATE_ENABLE	(1 << STA3XX_ConfE_SVE_OFFSET)	///< Volume adjustments will use soft volume
#define STA3XX_ConfE_SOFTVOLUMEUPDATE_DISABLE	(0 << STA3XX_ConfE_SVE_OFFSET)	///< Volume adjustments will occur immediatly

//ConfF
#define STA3XX_ConfF_OCFG				0
#define STA3XX_ConfF_IDE				2
#define STA3XX_ConfF_BCLE				3
#define STA3XX_ConfF_LRCK				4
#define STA3XX_ConfF_ECLE				5
#define STA3XX_ConfF_PWDN				6
#define STA3XX_ConfF_EAPD				7

#define STA3XX_ConfF_CONFIG_STEREO		(0 << STA3XX_ConfF_OCFG)			///< Stereo output (2 full-bridge channels)
#define STA3XX_ConfF_CONFIG_2_1			(1 << STA3XX_ConfF_OCFG)			///< 2.1 output (2 half-bridge channels + 1 full-bridge)
#define STA3XX_ConfF_CONFIG_2_DXX		(2 << STA3XX_ConfF_OCFG)			///< 2.1 output (2 half-bridge channels + 1 full-bridge)
#define STA3XX_ConfF_CONFIG_MONO		(3 << STA3XX_ConfF_OCFG)			///< Mono output (1 channel mono-parallel)

#define STA3XX_ConfF_INVALIDINPUTMUTE_DISABLE	(0 << STA3XX_ConfF_IDE)		///< Invalid input detect auto-mute disabled
#define STA3XX_ConfF_INVALIDINPUTMUTE_ENABLE	(1 << STA3XX_ConfF_IDE)		///< Invalid input detect auto-mute enabled

#define STA3XX_ConfF_CLOCKLOSSDETECT_DISABLE	(0 << STA3XX_ConfF_BCLE)	///< Binary output clock loss detection disabled
#define STA3XX_ConfF_CLOCKLOSSDETECT_ENABLE		(1 << STA3XX_ConfF_BCLE)	///< Binary output clock loss detection enabled

#define STA3XX_ConfF_LRCK_DBL_TRIGGER_DISABLE	(0 << STA3XX_ConfF_LRCK)	///< Binary output clock loss detection disabled
#define STA3XX_ConfF_LRCK_DBL_TRIGGER_ENABLE	(1 << STA3XX_ConfF_LRCK)	///< Binary output clock loss detection enabled

#define STA3XX_ConfF_AUTOEAPD_DISABLE			(0 << STA3XX_ConfF_ECLE)	///< Auto EAPD on clock loss disabled
#define STA3XX_ConfF_AUTOEAPD_ENABLE			(1 << STA3XX_ConfF_ECLE)	///< Auto EAPD on clock loss enabled

#define STA3XX_ConfF_NORMALOPERATION			(1 << STA3XX_ConfF_PWDN)	///< Normal operation
#define STA3XX_ConfF_POWERDOWN					(0 << STA3XX_ConfF_PWDN)	///< Powerdown mode operation (auto soft-mute enabled)

#define STA3XX_ConfF_EXTAMP_NORMALOPERATION		(1 << STA3XX_ConfF_EAPD)	///< Normal operation
#define STA3XX_ConfF_EXTAMP_POWERDOWN			(0 << STA3XX_ConfF_EAPD)	///< External power stage power down active


// Mute 
/////////
#define STA3XX_MASTERMUTE		1
#define STA3XX_C1MUTE			2
#define STA3XX_C2MUTE			4
#define STA3XX_C3MUTE			8
#define STA3XX_LOC_OFFSET		6

#define STA3XX_LOC_NOVOL_NOEQ		(0 << STA3XX_LOC_OFFSET)
#define STA3XX_LOC_VOL_NOEQ			(1 << STA3XX_LOC_OFFSET)
#define STA3XX_LOC_VOL_EQ			(2 << STA3XX_LOC_OFFSET)


// Auto-mode
//////////////

//AutoModes GC
#define STA3XX_AUTOMODE1_AMGC_OFFSET		4

#define STA3XX_AUTOMODE1_GAIN_USER			(0 << STA3XX_AUTOMODE1_AMGC_OFFSET)	///< User programmable gain compression/limiters
#define STA3XX_AUTOMODE1_GAIN_ACNOCLIPPING	(1 << STA3XX_AUTOMODE1_AMGC_OFFSET)	///< AC no clipping
#define STA3XX_AUTOMODE1_GAIN_ACLIMCLIPPING	(2 << STA3XX_AUTOMODE1_AMGC_OFFSET)	///< AC limited clipping
#define STA3XX_AUTOMODE1_GAIN_NIGHTMODE		(0 << STA3XX_AUTOMODE1_AMGC_OFFSET)	///< DRC nighttime listening mode

//AutoMode AM/pre-scale/bass management scale
#define STA3XX_AUTOMODE2_AMAM_OFFSET		0 /*AMAME + AMAM*/
#define STA3XX_AUTOMODE2_XO_OFFSET			4

#define STA3XX_AUTOMODE2_AMAM_DISABLE			(0 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM48K_0535_0720	(0x1 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM48K_0721_0900	(0x3 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM48K_0901_1100	(0x5 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM48K_1101_1300	(0x7 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM48K_1301_1480	(0x9 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM48K_1481_1600	(0xb << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM48K_1601_1700	(0xd << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM44K_0535_0670	(0x1 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM44K_0671_0800	(0x3 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM44K_0801_1000	(0x5 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM44K_1001_1180	(0x7 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM44K_1181_1340	(0x9 << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM44K_1341_1500	(0xb << STA3XX_AUTOMODE2_AMAM_OFFSET)
#define STA3XX_AUTOMODE2_AMAM44K_1501_1700	(0xd << STA3XX_AUTOMODE2_AMAM_OFFSET)

#define STA3XX_AUTOMODE2_CROSSOVER_USER		(0x0 << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_80HZ		(0x1 << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_100HZ	(0x2 << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_120HZ	(0x3 << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_140HZ	(0x4 << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_160HZ	(0x5 << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_180HZ	(0x6 << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_200HZ	(0x7 << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_220HZ	(0x8 << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_240HZ	(0x9 << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_260HZ	(0xa << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_280HZ	(0xb << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_300HZ	(0xc << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_320HZ	(0xd << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_340HZ	(0xe << STA3XX_AUTOMODE2_XO_OFFSET)
#define STA3XX_AUTOMODE2_CROSSOVER_360HZ	(0xf << STA3XX_AUTOMODE2_XO_OFFSET)

// Channel configuration
//////////////////////////

#define STA3XX_CHANNEL_TCB_OFFSET		0
#define STA3XX_CHANNEL_EQBP_OFFSET		1
#define STA3XX_CHANNEL_VBP_OFFSET		2
#define STA3XX_CHANNEL_BO_OFFSET		3
#define STA3XX_CHANNEL_LS_OFFSET		4
#define STA3XX_CHANNEL_OM_OFFSET		6

#define STA3XX_CHANNEL_TONECTRL_ENABLE	(0 << STA3XX_CHANNEL_TCB_OFFSET)	///< Perform tone control on channel
#define STA3XX_CHANNEL_TONECTRL_BYPASS	(1 << STA3XX_CHANNEL_TCB_OFFSET)	///< Bypass tone control on channel

#define STA3XX_CHANNEL_EQ_ENABLE		(0 << STA3XX_CHANNEL_EQBP_OFFSET)	///< Perform EQ on channel
#define STA3XX_CHANNEL_EQ_BYPASS		(1 << STA3XX_CHANNEL_EQBP_OFFSET)	///< Bypass EQ on channel

#define STA3XX_CHANNEL_PWM_DDX			(0 << STA3XX_CHANNEL_BO_OFFSET)		///< Output DDX PWM data
#define STA3XX_CHANNEL_PWM_BINARY		(1 << STA3XX_CHANNEL_BO_OFFSET)		///< Output standard binary PWM encoded data

#define STA3XX_CHANNEL_LIMITER_DISABLE	(0 << STA3XX_CHANNEL_LS_OFFSET)		///< Channel has limiter disabled
#define STA3XX_CHANNEL_LIMITER_1		(1 << STA3XX_CHANNEL_LS_OFFSET)		///< Channel is mapped to limiter #1
#define STA3XX_CHANNEL_LIMITER_2		(2 << STA3XX_CHANNEL_LS_OFFSET)		///< Channel is mapped to limiter #2

#define STA3XX_CHANNEL_OUTPUTMAP_CHAN1	(0 << STA3XX_CHANNEL_OM_OFFSET)		///< PWM output from Channel 1
#define STA3XX_CHANNEL_OUTPUTMAP_CHAN2	(1 << STA3XX_CHANNEL_OM_OFFSET)		///< PWM output from Channel 2
#define STA3XX_CHANNEL_OUTPUTMAP_CHAN3	(2 << STA3XX_CHANNEL_OM_OFFSET)		///< PWM output from Channel 3


// Tone control
/////////////////

#define STA3XX_TONE_BASS_OFFSET			0
#define STA3XX_TONE_TREBBLE_OFFSET		4

/** Bass boost/cut.
 * x in the range -12 to +12 (dB), with 2dB step.
 */
#define STA3XX_TONE_BASS(x)					((7 + (x)/2) << STA3XX_TONE_BASS_OFFSET)

/** Trebble boost/cut.
 * x in the range -12 to +12 (dB), with 2dB step.
 */
#define STA3XX_TONE_TREBBLE(x)				((7 + (x)/2) << STA3XX_TONE_TREBBLE_OFFSET)

//Coefficient
///////////////

#define STA3XX_Coeff_ReadAll		8
#define STA3XX_Coeff_ReadOne		4
#define STA3XX_Coeff_WriteAll		2
#define STA3XX_Coeff_WriteOne		1

enum STA3XX_Coeff_BIQUAD_TAG {
	STA3XX_Coeff_CHAN1BIQUAD1 = 0x00,
	STA3XX_Coeff_CHAN1BIQUAD2 = 0x05,
	STA3XX_Coeff_CHAN1BIQUAD3 = 0x0a,
	STA3XX_Coeff_CHAN1BIQUAD4 = 0x0f,
	STA3XX_Coeff_CHAN2BIQUAD1 = 0x14,
	STA3XX_Coeff_CHAN2BIQUAD2 = 0x19,
	STA3XX_Coeff_CHAN2BIQUAD3 = 0x1e,
	STA3XX_Coeff_CHAN2BIQUAD4 = 0x23,
	STA3XX_Coeff_HIGHPASSFILTER = 0x28,
	STA3XX_Coeff_LOWPASSFILTER = 0x2d,
};
typedef enum STA3XX_Coeff_BIQUAD_TAG STA3XX_Coeff_BIQUAD;

enum STA335BX_MIXCOEFF_TAG
{
	STA3XX_MIXCOEFF_CHAN1_MIX1 = 0x38,
	STA3XX_MIXCOEFF_CHAN1_MIX2 = 0x39,
	STA3XX_MIXCOEFF_CHAN2_MIX1 = 0x3a,
	STA3XX_MIXCOEFF_CHAN2_MIX2 = 0x3b,
	STA3XX_MIXCOEFF_CHAN3_MIX1 = 0x3c,
	STA3XX_MIXCOEFF_CHAN3_MIX2 = 0x3d,
};
typedef enum STA335BX_MIXCOEFF_TAG STA335BX_MIXCOEFF;

enum STA3XX_Coeff_TAG {
	STA3XX_Coeff_CHAN1BIQ1_B1,
	STA3XX_Coeff_CHAN1BIQ1_B2,
	STA3XX_Coeff_CHAN1BIQ1_A1,
	STA3XX_Coeff_CHAN1BIQ1_A2,
	STA3XX_Coeff_CHAN1BIQ1_B0,
	
	STA3XX_Coeff_CHAN1BIQ2_B1,
	STA3XX_Coeff_CHAN1BIQ2_B2,
	STA3XX_Coeff_CHAN1BIQ2_A1,
	STA3XX_Coeff_CHAN1BIQ2_A2,
	STA3XX_Coeff_CHAN1BIQ2_B0,
	
	STA3XX_Coeff_CHAN1BIQ3_B1,
	STA3XX_Coeff_CHAN1BIQ3_B2,
	STA3XX_Coeff_CHAN1BIQ3_A1,
	STA3XX_Coeff_CHAN1BIQ3_A2,
	STA3XX_Coeff_CHAN1BIQ3_B0,
	
	STA3XX_Coeff_CHAN1BIQ4_B1,
	STA3XX_Coeff_CHAN1BIQ4_B2,
	STA3XX_Coeff_CHAN1BIQ4_A1,
	STA3XX_Coeff_CHAN1BIQ4_A2,
	STA3XX_Coeff_CHAN1BIQ4_B0,
};
typedef enum STA3XX_Coeff_TAG STA3XX_Coeff;

typedef struct STA3XX_CONFIG_TAG{
	cyg_uint8 confA;
	cyg_uint8 confB;
	cyg_uint8 confC;
	cyg_uint8 confD;
	cyg_uint8 confE;
	cyg_uint8 confF;

	cyg_uint8 volumeMaster;

	cyg_uint8 volumeChan1;
	cyg_uint8 volumeChan2;
	cyg_uint8 volumeChan3;
} STA3XX_CONFIG;

void STA3XX_Init(STA3XX_DEVICE device, cyg_i2c_bus* bus, const STA3XX_CONFIG * config);
		
void STA3XX_Write(STA3XX_DEVICE device, cyg_uint16 addr, const cyg_uint8 * buf, cyg_uint8 len);
void STA3XX_Read(STA3XX_DEVICE device, cyg_uint16 addr, cyg_uint8 * buf , cyg_uint8 len );

static inline void STA3XX_Write1(STA3XX_DEVICE device, cyg_uint16 addr, cyg_uint8 value)		{ STA3XX_Write(device, addr, &value, 1); }
static inline int STA3XX_Read1(STA3XX_DEVICE device, cyg_uint16 addr)	{ cyg_uint8 value; STA3XX_Read(device, addr, &value, 1); return value; }

void STA3XX_Coeff_Write(STA3XX_DEVICE device, STA3XX_Coeff coeff, int value);
void STA3XX_Coeff_Read(STA3XX_DEVICE device, STA3XX_Coeff coeffadd, int *value);
void STA3XX_Coeff_WriteBiquad(STA3XX_DEVICE device, STA3XX_Coeff_BIQUAD biquad, int b1, int b2, int a1, int a2, int b0);

void STA3XX_Mute(STA3XX_DEVICE device, cyg_uint8 mute);
void STA3XX_UnMute(STA3XX_DEVICE device, cyg_uint8 mute);

void STA3XX_InitSimple(STA3XX_DEVICE device, cyg_i2c_bus *bus);

#define STA3XX_MUTE(device, muted)				(muted ? STA3XX_Mute(device, STA3XX_MASTERMUTE) : STA3XX_UnMute(device, STA3XX_MASTERMUTE))
																				   
#define STA3XX_SET_VOLUME(device, vol)			STA3XX_Write1(device, STA3XX_REG_MasterVolume, vol)
#define STA3XX_SET_CHAN1_VOLUME(device, vol)		STA3XX_Write1(device, STA3XX_REG_Chan1Volume, vol)
#define STA3XX_SET_CHAN2_VOLUME(device, vol)		STA3XX_Write1(device, STA3XX_REG_Chan2Volume, vol)
#define STA3XX_SET_CHAN3_VOLUME(device, vol)		STA3XX_Write1(device, STA3XX_REG_Chan3Volume, vol)

#endif /*_STA3XX_*/
