/**
 ****************************************************************************
 * Driver for Wolfson WM8960 stereo audio codec.
 *
 * @file   include/wm8960_itf.h
 * @brief  Wolfson WM8960 API Header.
 *
 * Code is based on the WM8960.pdf in the doc directory of this package
 *
 * @author Gregoire ETIENNE
 * @date   2007-03-23
 *
 * Copyright (C) 2007, Parrot S.A. - www.parrot.biz
 ****************************************************************************
 */

/**
 * @addtogroup wm8960_api API of the WM8960 Driver 
 *
 * @{
 */


extern void codec_wm8960_init(void);

/** @} */








/**
 *@mainpage  WM8960 audio codec driver for eCos
 *
 * This doxygen-generated documentation explains the internals of WM8960 audio
 * codec driver, both for the application implementing the driver and also for
 * driver maintaining.
 *
 * @section howtouse_doc  Wolfson WM8960 Datasheet
 * 
 * Please refer to the document below, available on CVS in the eCos repository:
 * <a href='http://canari/cgi-bin/cvsweb/~checkout~/SoftParrot5/ecos/ecos-current/packages/devs/i2c/components/wm8960/current/doc/WM8960.pdf'> WM8960.pdf</a>
*
* \section ecos_config eCos Configuration
*
* This package requires to load i2c packages : "Generic I2C support" & "Parrot5 I2C master driver". Otherwise, it wouldn't be active.
* The i2c bus speed that will be used is the default one configurated in the configtool
*
\section code_sec Code example
A coding example :
* <ul>
* <li>\ref code1_sec
* </ul>
*
*
*/




/*! \page page_examples Code example

\section code1_sec Loopback
The following is a run-through of a program that takes a stream from an input 
channel <tt>/dev/aai/mic12</tt> and writes it directly to an output channel <tt>/dev/aai/hp</tt>.

First, relevant headers have to be included.
\code
#include <cyg/io/io.h>
#include <cyg/io/aai.h>
#include <stdio.h>          // needed for fprintf()
#include <cyg/kernel/kapi.h>
\endcode

Next, two buffers are declared.
\code
cyg_int16 buffer[2][128];
\endcode

Inside our thread code, we first lookup relevant devices. Variables other than
<tt>status</tt>, <tt>ext_in</tt> and <tt>hp</tt> will be used later.
\code
void play(void)
{
   int len;
   int k;
   cyg_io_handle_t hp, mic12;
   int status;
   int set_dsp = 1;

   status = cyg_io_lookup("/dev/aai/hp", &hp);
   if ( status != ENOERR ) 
   {
      fprintf( stderr, "Failed to lookup device /dev/aai/hp. Aborting...\n" );
      return;
   }

   status = cyg_io_lookup("/dev/aai/mic12", &mic12);
   if ( status != ENOERR ) 
   {
      fprintf( stderr, "Failed to lookup device /dev/aai/mic12. Aborting...\n" );
      return;
   }

   ...
}
\endcode

We enable <b>DSP MODE</b> because we want the stream to be on stereo @ 44,1 kHz. 
This is the old Parrot5 way to modify sample rate. On the Parrot5+, it can be done
Independantly.
\code
   len = sizeof(set_dsp);
   cyg_io_set_config(mic12, CYG_IO_SET_CONFIG_AAI_DSP_MODE, &set_dsp, &len);
\endcode

Then we enter an infinite loop, in which we alternatively swap buffers.
When <tt>cyg_io_read()</tt> on <tt>buffer[0]</tt> has returned,
we send out <tt>buffer[1]</tt> to <tt>/dev/aai/hp</tt> and request reading 
on <tt>buffer[1]</tt>. Once <tt>cyg_io_read()</tt> on <tt>buffer[1]</tt>
has returned, we send out <tt>buffer[0]</tt>, and so on...
\code
   k=0;
   do 
   {
      len = 128*sizeof(char);
      status = cyg_io_read( mic12, &(buffer[k][0]), &len );
      if ( status != ENOERR ) 
      {
         diag_printf( "Failed to read device /dev/aai/mic12. Aborting...\n" );
         return;
      }
      k = (k+1)%2; // swap buffers
      len = 128*sizeof(char);
      status = cyg_io_write( hp, &(buffer[k][0]), &len );
      if (status != ENOERR)
      {
         diag_printf( "Failed to write device /dev/aai/hp. Aborting...\n" );
         return;
      }

   }while(1) ;
\endcode

Finally some code is necessary to launch our audio thread.
\code
cyg_thread thread;
char stack[4096];
cyg_handle_t thread_handle;

void cyg_user_start(void) {
cyg_thread_create( 4, play, (cyg_addrword_t)0, "WM8960 test", (void *)stack,
4096, &thread_handle, &thread );
cyg_thread_resume(thread_handle);
}
\endcode

*/

