/**
 ****************************************************************************
 * Driver for Wolfson WM8960 stereo audio codec.
 *
 * @file   include/wm8960_regs.h
 * @brief  Wolfson WM8960 Registers Description.
 *
 * Code is based on the WM8960.pdf in the doc directory of this package
 *
 * @author Gregoire ETIENNE
 * @date   2007-03-23
 *
 * Copyright (C) 2007, Parrot S.A. - www.parrot.biz
 ****************************************************************************
 */

#ifndef WM8960_H
#define WM8960_H



#define WM8960_ADDRESS   0x1A //!< WM8960 Codec Adress on the i2c bus

#define CODEC_R0_LIN_VOL        0x00 //!< 
#define CODEC_R1_RIN_VOL        0x01 //!<
#define CODEC_R2_LOUT_VOL       0x02 //!<
#define CODEC_R3_ROUT_VOL       0x03 //!<
#define CODEC_R4_CLOCKING1      0x04 //!<
#define CODEC_R5_ADC_DAC_CTRL1  0x05 //!<
#define CODEC_R6_ADC_DAC_CTRL2  0x06 //!<
#define CODEC_R7_AUDIO_ITF1     0x07 //!<
#define CODEC_R8_CLOCKING2      0x08 //!<
#define CODEC_R9_AUDIO_ITF2     0x09 //!<
#define CODEC_R10_LDAC_VOL      0x0A //!<
#define CODEC_R11_RDAC_VOL      0x0B //!<
#define CODEC_R12_RSVD          0x0C //!< Reserved
#define CODEC_R13_RSVD          0x0D //!< Reserved
#define CODEC_R14_RSVD          0x0E //!< Reserved
#define CODEC_R15_RESET         0x0F //!<
#define CODEC_R16_3D_CTRL       0x10 //!<
#define CODEC_R17_ALC1          0x11 //!<
#define CODEC_R18_ALC2          0x12 //!<
#define CODEC_R19_ALC3          0x13 //!<
#define CODEC_R20_NOISE_GATE    0x14 //!<
#define CODEC_R21_LADC_VOL      0x15 //!<
#define CODEC_R22_RADC_VOL      0x16 //!<
#define CODEC_R23_ADDIT_CTRL1   0x17 //!<
#define CODEC_R24_ADDIT_CTRL2   0x18 //!<
#define CODEC_R25_POWER_MNGT1   0x19 //!<
#define CODEC_R26_POWER_MNGT2   0x1A //!<
#define CODEC_R27_ADDIT_CTRL3   0x1B //!<
#define CODEC_R28_ANTI_POP1     0x1C //!<
#define CODEC_R29_ANTI_POP2     0x1D //!<
#define CODEC_R30_RSVD          0x1B //!< Reserved
#define CODEC_R31_RSVD          0x1C //!< Reserved
#define CODEC_R32_ADCL_SIGPATH  0x20 //!<
#define CODEC_R33_ADCR_SIGPATH  0x21 //!<
#define CODEC_R34_LOUT_MIX      0x22 //!<
#define CODEC_R35_RSVD          0x23 //!< Reserved
#define CODEC_R36_RSVD          0x24 //!< Reserved
#define CODEC_R37_ROUT_MIX      0x25 //!<
#define CODEC_R38_MONO_OUT_MIX1 0x26 //!<
#define CODEC_R39_MONO_OUT_MIX2 0x27 //!<
#define CODEC_R40_L_SPK_VOL     0x28 //!<
#define CODEC_R41_R_SPK_VOL     0x29 //!<
#define CODEC_R42_OUT3_VOL      0x2A //!<
#define CODEC_R43_LINBSTMIX     0x2B //!<
#define CODEC_R44_RINBSTMIX     0x2C //!<
#define CODEC_R45_L_BYPASS      0x2D //!<
#define CODEC_R46_R_BYPASS      0x2E //!<
#define CODEC_R47_POWER_MNGT3   0x2F //!<
#define CODEC_R48_ADDIT_CTRL4   0x30 //!<
#define CODEC_R49_CLASS_D_CTRL1 0x31 //!<
#define CODEC_R50_RSVD          0x32 //!< Reserved
#define CODEC_R51_CLASS_D_CTRL2 0x33 //!<
#define CODEC_R52_PLL1          0x34 //!<
#define CODEC_R53_PLL2          0x35 //!<
#define CODEC_R54_PLL3          0x36 //!<
#define CODEC_R55_PLL4          0x37 //!<

/**
 * @defgroup wm8960_regs_descr  Register description
 *
 * Here is the description of register contents, i.e. what does each bit
 * mean in each register.
 *
 * @{
 */

/**
 * @defgroup wm8960_regs_r0  R0(0x00): Left Input Volume
 * @{
 */
#define IPVU           (1<<8)   //!< Left Input PGA Volume update
#define LINMUTE        (1<<7)   //!< Left Input PGA Analogue Mute
#define LIZC           (1<<6)   //!< Left Input PGA Zero Cross Detector
#define LINVOL(_i_)    (_i_)    //!< Left Input PGA Volume level
/** @}
 * @defgroup wm8960_regs_r1  R1(0x01):  Right Input Volume
 * @{
 */
#define IPVU           (1<<8)    //!< Right Input PGA Volume update
#define RINMUTE        (1<<7)    //!< Right Input PGA Analogue Mute
#define RIZC           (1<<6)    //!< Right Input PGA Zero Cross Detector
#define RINVOL(_i_)    (_i_)     ///< Right Input PGA Volume level
/** @}
 * @defgroup wm8960_regs_r2  R2(0x02): Left Output Volume
 * @{
 */
#define OUT1VU         (1<<8)    //!< Headphone Output PGA Output Volume update
#define LOZC           (1<<7)    //!< Left Headphone Output Zero Cross Enable
#define LOUT1VOL(_i_)  (_i_)     ///< Left Output Volume level
/** @}
 * @defgroup wm8960_regs_r3  R2(0x02): Right Output Volume
 * @{
 */
#define OUT1VU         (1<<8)    //!< Headphone Output PGA Output Volume update
#define LOZC           (1<<7)    //!< Right Headphone Output Zero Cross Enable
#define ROUT1VOL(_i_)  (_i_)     ///< Right Output Volume level
/** @}
 * @defgroup wm8960_regs_r5  R5(0x05): ADC & DAC Control 1
 * @{
 */
#define DACMU          (1<<3)    //!< DAC Mute
/** @}
 * @defgroup wm8960_regs_r9  R9(0x09): Audio Interface
 * @{
 */
#define LOOPBACK       (1)       //!< Loopback enabled, ADC data output is fed directly into DAC
/** @}
 * @defgroup wm8960_regs_r25  R25(0x19): Power Management 1
 * @{
 */
#define VMIDSEL(_i_)   (_i_<<7)  //!< VMID Divider Enable And Select
#define VREF           (1<<6)    //!< Power Switch
#define AINL           (1<<5)    //!< Enable Analogue in PGA Left
#define AINR           (1<<4)    //!< Enable Analogue in PGA Right
#define ADCL           (1<<3)    //!< Enable Left ADC
#define ADCR           (1<<2)    //!< Enable Right ADC
#define MICB           (1<<1)    //!< Enable Microphone Bias

/** @}
 * @defgroup wm8960_regs_r26  R26(0x1A): Power Management 2
 * @{
 */
#define DACL           (1<<8)    //!< Enable Left DAC
#define DACR           (1<<7)    //!< Enable Right DAC
#define LOUT1          (1<<6)    //!< Enable Left Headphone Output
#define ROUT1          (1<<5)    //!< Enable Right Headphone Output 
#define SPKL           (1<<4)    //!< Enable Left Speaker Output 
#define SPKR           (1<<3)    //!< Enable Right Speaker Output
#define OUT3           (1<<1)    //!< Enable OUT3 Output (used as ground)

/** @}
 * @defgroup wm8960_regs_r32  R32(0x20): ADC Left Signal Path
 * @{
 */
#define LMN1            (1<<8)   //!< Connect Left Input 1 to inverting input of left input PGA
#define LMP3            (1<<7)   //!< Connect Left Input 3 to non-inverting input of left input PGA
#define LMP2            (1<<6)   //!< Connect Left Input 2 to non-inverting input of left input PGA
#define LMICBOOST_0DB   (0x0<<4) //!< Left Channel Input PGA Boost Gain is  0dB
#define LMICBOOST_13DB  (0x1<<4) //!< Left Channel Input PGA Boost Gain is 13dB
#define LMICBOOST_20DB  (0x2<<4) //!< Left Channel Input PGA Boost Gain is 20dB
#define LMICBOOST_29DB  (0x3<<4) //!< Left Channel Input PGA Boost Gain is 29dB
#define LMIC2B          (1<<3)   //!< Connect Left Input PGA to Left Input Boost Mixer
/** @}
 * @defgroup wm8960_regs_r33  R33(0x21): ADC Right Signal Path
 * @{
 */
#define RMN1            (1<<8)   //!< Connect Right Input 1 to inverting input of left input PGA
#define RMP3            (1<<7)   //!< Connect Right Input 3 to non-inverting input of left input PGA
#define RMP2            (1<<6)   //!< Connect Right Input 2 to non-inverting input of left input PGA
#define RMICBOOST_0DB   (0x0<<4) //!< Right Channel Input PGA Boost Gain is  0dB
#define RMICBOOST_13DB  (0x1<<4) //!< Right Channel Input PGA Boost Gain is 13dB
#define RMICBOOST_20DB  (0x2<<4) //!< Right Channel Input PGA Boost Gain is 20dB
#define RMICBOOST_29DB  (0x3<<4) //!< Right Channel Input PGA Boost Gain is 29dB
#define RMIC2B          (1<<3)   //!< Connect Right Input PGA to Right Input Boost Mixer
/** @}
 * @defgroup wm8960_regs_r34  R34(0x22): Left Out Mix 1
 * @{
 */
#define LD2LO           (1<<8)   //!< Left DAC to Left Output Mixer
#define LI2LO           (1<<7)   //!< Left Input 3 to Left Output Mixer
/** @}
 * @defgroup wm8960_regs_r37  R37(0x25): Right Out Mix 1
 * @{
 */
#define RD2RO           (1<<8)   //!< Right DAC to Right Output Mixer
#define RI2RO           (1<<7)   //!< Right Input 3 to Right Output Mixer
/** @}
 * @defgroup wm8960_regs_r40  R40(0x28): Left Out Speaker Volume
 * @{
 */
#define SPKVU           (1<<8)   //!< Left Speaker Volume Update
#define SPKLZC          (1<<7)   //!< Left Speaker Zero Cross Enable
#define SPKLVOL(_i_)    (_i_)    //!< Left Speaker Volume
/** @}
 * @defgroup wm8960_regs_r41  R41(0x29): Right Out Speaker Volume
 * @{
 */
#define SPKVU           (1<<8)   //!< Right Speaker Volume Update
#define SPKRZC          (1<<7)   //!< Right Speaker Zero Cross Enable
#define SPKRVOL(_i_)    (_i_)    //!< Right Speaker Volume
/** @}
 * @defgroup wm8960_regs_r45  R45(0x2D): Bypass Left
 * @{
 */
#define LB2LO           (1<<7)   //!< Left Input Boost Mixer to Left Output Mixer
#define LB2LOVOL(_i_)   (_i_<<4) //!< Left Input Boost Mixer to Left Output Mixer Volume
/** @}
 * @defgroup wm8960_regs_r46  R46(0x2E): Bypass Right
 * @{
 */
#define RB2RO           (1<<7)   //!< Right Input Boost Mixer to Right Output Mixer
#define RB2ROVOL(_i_)   (_i_<<4) //!< Right Input Boost Mixer to Right Output Mixer Volume
/** @}
 * @defgroup wm8960_regs_r47  R47(0x2F): Power Management 3
 * @{
 */
#define LMIC            (1<<5)   //!< Left Input PGA Enable
#define RMIC            (1<<4)   //!< Right Input PGA Enable
#define LOMIX           (1<<3)   //!< Left Output Mixer Enable Control
#define ROMIX           (1<<2)   //!< Right Output Mixer Enable Control
/** @}
 * @defgroup wm8960_regs_r49  R49(0x31): Class D Control
 * @{
 */
#define SPKOPEN(_i_)    (_i_<<6) //!< Enable Class D Speaker Outputs
/** @} */
// !wm8960_regs_descr
/** @} */




/** \struct wm8960_config_tab_t
 * \brief Contains a register configuration.
 *
 * This struct contains the address of a register and the value
 * to write into.
 *
 *  \li <b>addr</b> : Address register (coded on 7 bits)
 *  \li <b>val</b> : Value of the register (coded on 9 bits)
 *
 */
typedef struct{
   cyg_uint8  addr;
   cyg_uint16 val;
} wm8960_config_tab_t;

#endif

