/**
 ***************************************************************************
 * @file wm8974.h
 *
 * @brief WM8974 digital amplifier driver.
 *
 * Copyright (C) 2006 Parrot S.A.
 *
 * @author	francois.ferrand@parrot.fr
 * @author	jerome.leveque@parrot.fr
 * @date	07/2006
 ***************************************************************************
 */

#ifndef _WM8974_
#define _WM8974_

#include <cyg/io/io.h>
#include <cyg/io/i2c.h>

// Registers
//////////////

/** WM8974 registers
 */
enum WM8974_{
	WM8974_REG_SOFT_RESET			= 0x00, 
	WM8974_REG_POWER_MNG_1			= 0x01,	
	WM8974_REG_POWER_MNG_2			= 0x02,	
	WM8974_REG_POWER_MNG_3			= 0x03,	

	WM8974_REG_AUDIO_INT			= 0x04,	
	WM8974_REG_COMPAND_CTRL			= 0x05,	
	WM8974_REG_CLOCK_GEN_CTRL		= 0x06,	

	WM8974_REG_ADD_CTRL				= 0x07,	
	WM8974_REG_GPIO_STUFF			= 0x08,	

	WM8974_REG_DAC_CTRL				= 0x0A,	
	WM8974_REG_DAC_DIGI_VOL			= 0x0B,	

	WM8974_REG_ADC_CTRL				= 0x0E,	
	WM8974_REG_ADC_DIGI_VOL			= 0x0F,	
	
	WM8974_REG_EQ_LOW_SHELF			= 0x12,
	WM8974_REG_EQ_PEAK_1			= 0x13,
	WM8974_REG_EQ_PEAK_2			= 0x14,
	WM8974_REG_EQ_PEAK_3			= 0x15,
	WM8974_REG_EQ_HIGH_SHELF		= 0x16,

	WM8974_REG_DAC_LIMITER_1		= 0x18,  
	WM8974_REG_DAC_LIMITER_2		= 0x19,  

	WM8974_REG_NOTCH_FILTER_1		= 0x1B,  
	WM8974_REG_NOTCH_FILTER_2		= 0x1C,  
	WM8974_REG_NOTCH_FILTER_3		= 0x1D,  
	WM8974_REG_NOTCH_FILTER_4		= 0x1E,  

	WM8974_REG_ALC_CTRL_1			= 0x20,  
	WM8974_REG_ALC_CTRL_2			= 0x21,  
	WM8974_REG_ALC_CTRL_3			= 0x22,  

	WM8974_REG_NOISE_GATE			= 0x23,  
	
	WM8974_REG_PLL_N				= 0x24,  
	WM8974_REG_PLL_K1				= 0x25,  
	WM8974_REG_PLL_K2				= 0x26,  
	WM8974_REG_PLL_K3				= 0x27,  

	WM8974_REG_INPUT_CTRL			= 0x2C,
	WM8974_REG_INPUT_PGA_GAIN_CTRL	= 0x2D,
	WM8974_REG_ADC_BOOST_CTRL		= 0x2F,

	WM8974_REG_OUTPUT_CTRL			= 0x31,

	WM8974_REG_SPK_MIXER_CTRL		= 0x32,
	WM8974_REG_SPK_VOLUME_CTRL		= 0x36,
	WM8974_REG_MONO_MIXER_CTRL		= 0x38,
};

cyg_bool WM8974_Init(cyg_i2c_bus* bus);
		
cyg_bool WM8974_Write(cyg_uint8 addr, const cyg_uint16 data);
//void WM8974_Read(cyg_uint8 addr, cyg_uint8 * buf , cyg_uint8 len );

//static inline void WM8974_Write1(cyg_uint16 addr, cyg_uint8 value)	{ WM8974_Write(addr, &value, 1); }
//static inline int WM8974_Read1(cyg_uint16 addr)					{ cyg_uint8 value; WM8974_Read(addr, &value, 1); return value; }

//void WM8974_Coeff_Write(WM8974_Coeff coeff, int value);
//void WM8974_Coeff_WriteBiquad(WM8974_Coeff_BIQUAD biquad, int b1, int b2, int a1, int a2, int b0);

#endif /*_WM8974_*/

