//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
* @file io/serial/arm/parrot5_serial.c
* @brief Parrot5/5+ serial driver
*
* Copyright (C) 2005,2006,2007 Parrot S.A.
*
* @author     ivan.djelic@parrot.fr
* @date       2005-09-28
*******************************************************************************
*/

#include <pkgconf/system.h>
#include <cyg/hal/cpu_types.h>
#include <pkgconf/io_serial.h>
#include <pkgconf/io.h>

#include <cyg/io/io.h>
#include <cyg/hal/hal_intr.h>           // interrupt
#include <cyg/io/devtab.h>
#include <cyg/io/serial.h>
#include <cyg/infra/diag.h>
#include <cyg/infra/cyg_ass.h>
#include <cyg/hal/hal_io.h>

#if defined(CYGPKG_HAL_ARM_ARM9_P5DEV)
#include <pkgconf/hal_arm.h>
#include <cyg/hal/p5dev.h>          // platform definitions
#elif defined(CYGPKG_HAL_ARM_ARM9_P6DEV)
#include <pkgconf/hal_arm.h>
#include <cyg/hal/p6dev.h>          // platform definitions
#elif defined(CYGPKG_HAL_CORTEXA_P7)
#include <cyg/hal/p7.h>             // platform definitions
#endif

// Only compile driver if an inline file with driver details was selected.
#ifdef CYGDAT_IO_SERIAL_PARROT5_INL

// FIFO sizes
#define UART_TX_FIFO_SIZE_P5        (16)
#define UART_RX_FIFO_SIZE_P5        (32)
#define UART_TX_FIFO_SIZE_P5P       (32)
#define UART_RX_FIFO_SIZE_P5P       (64)
#define UART_TX_FIFO_SIZE_P7        (64)
#define UART_RX_FIFO_SIZE_P7        (64)

//==========================================================================

// Selectable baud rates
static const unsigned int select_baud[CYGNUM_SERIAL_BAUD_MAX + 1] = {
    0,
    50,
    75,
    110,
    0,
    150,
    200,
    300,
    600,
    1200,
    1800,
    2400,
    3600,
    4800,
    7200,
    9600,
    14400,
    19200,
    38400,
    57600,
    115200,
    230400,
    460800,
    921600,
    1500000,
    2000000,
    3000000,
};

// Stop bits
static const unsigned char select_stop_bits[] = {
    0,                   // unused
    0,                   // 1 stop bit
    0,                   // 1.5 stop bit (not supported)
    UART_LCR_TWOSTOP,    // 2 stop bits
};

static const unsigned char select_word_length[] = {
    0,                   // 5 bits (not supported)
    0,                   // 6 bits (not supported)
    0,                   // 7 bits
    UART_LCR_EIGHTBITS,  // 8 bits
};

static const unsigned char select_parity[] = {
    0,                                              // No parity
    UART_LCR_PARITY|UART_LCR_EVEN,                  // Even parity
    UART_LCR_PARITY,                                // Odd parity
    UART_LCR_PARITY|UART_LCR_STICKY,                // Mark parity  (1)
    UART_LCR_PARITY|UART_LCR_STICKY|UART_LCR_EVEN   // Space parity (0)
};

//==========================================================================

typedef struct parrot5_serial_info {
    CYG_ADDRWORD   base;
    cyg_uint32     clock_en;
    CYG_WORD       int_num;
    int            txfill;
    cyg_interrupt  serial_interrupt;
    cyg_handle_t   serial_interrupt_handle;
} parrot5_serial_info;

static bool
parrot5_serial_init(struct cyg_devtab_entry *tab);
static bool
parrot5_serial_putc(serial_channel *chan, unsigned char c);
static Cyg_ErrNo
parrot5_serial_lookup( struct cyg_devtab_entry **tab,
                       struct cyg_devtab_entry *sub_tab,
                       const char *name);
static unsigned char
parrot5_serial_getc(serial_channel *chan);
static Cyg_ErrNo
parrot5_serial_set_config(serial_channel *chan, cyg_uint32 key,
                          const void *xbuf, cyg_uint32 *len);
static void
parrot5_serial_start_xmit(serial_channel *chan);
static void
parrot5_serial_stop_xmit(serial_channel *chan);

static cyg_uint32
parrot5_serial_isr(cyg_vector_t vector, cyg_addrword_t data);
static void
parrot5_serial_dsr(cyg_vector_t vector, cyg_ucount32 count,
                   cyg_addrword_t data);

static SERIAL_FUNS(parrot5_serial_funs,
                   parrot5_serial_putc,
                   parrot5_serial_getc,
                   parrot5_serial_set_config,
                   parrot5_serial_start_xmit,
                   parrot5_serial_stop_xmit );

//==========================================================================

#include CYGDAT_IO_SERIAL_PARROT5_INL

// Parrot5+ specific stuff
// define TX fifo threshold - make sure these two defines below are consistent
#define TX_FIFO_THRESHOLD                (4)
#define LCR_TX_FIFO_THRESHOLD            (UART_LCR_FIFO_TX_THRESHOLD_4)

#define CYG_IO_SERIAL_PARROT5_INT_PRIORITY 4

//==========================================================================
// Internal function to actually configure the hardware to desired
// baud rate, etc.

static bool
serial_config_port( serial_channel *chan, cyg_serial_info_t *new_cfg,
                    bool init ) {
    cyg_uint32 baud, baud_divisor, lcr;
    parrot5_serial_info *p5_chan = (parrot5_serial_info *)chan->dev_priv;
    CYG_ADDRWORD base = p5_chan->base;

    if ( init ) {
        // start UART clock in system controller
#if defined(CYGPKG_HAL_ARM_ARM9_P6DEV)
        HAL_WRITE_UINT32(PARROT6_SYS+_P6_SYS_CEN, p5_chan->clock_en);
#elif defined(CYGPKG_HAL_ARM_ARM9_P5DEV)
        HAL_WRITE_UINT32( PARROT5_SYS+_SYS_CEN, p5_chan->clock_en );
#elif defined(CYGPKG_HAL_CORTEXA_P7)
#warning TODO
#endif
    }
    // first stop and reset UART
    HAL_WRITE_UINT32( base+_UART_SPD, 0);

    // a baudrate of 0 is OK, it means we just want to stop the device
    if ( new_cfg->baud ) {

        // baud rates <= CYGNUM_SERIAL_BAUD_MAX are indexes in a table,
        // baud rates > CYGNUM_SERIAL_BAUD_MAX are real baud rates
        baud = ( new_cfg->baud <= CYGNUM_SERIAL_BAUD_MAX )?
            select_baud[new_cfg->baud] : new_cfg->baud;

        // check baudrate
        if ( !baud ||( baud < 16 )) {
            return false;
        }
#if defined(CYGPKG_HAL_ARM_ARM9_P6DEV)
        // querying the proper uart clock would be cleaner
        baud_divisor = (pclock_get_rate(NULL) + baud/2) / baud;
#elif defined(CYGPKG_HAL_ARM_ARM9_P5DEV)
        // compute baud divisor, this depends on whether we are running at
        // 208 MHz, 156 MHz or 260 MHz (P5+)
        baud_divisor = P5_CLOCK_ADJUST(PARROT5_UART_CLOCK)/baud;
#elif defined(CYGPKG_HAL_CORTEXA_P7)
#warning TODO
#endif
        lcr =
            select_word_length[ new_cfg->word_length -
                                CYGNUM_SERIAL_WORD_LENGTH_5]|
            select_stop_bits[new_cfg->stop]|
            select_parity[new_cfg->parity];

        // enable TX and RX FIFOs
        if ( chan->out_cbuf.len != 0 ) {

            switch (get_cpu_types())
            {
                case PARROT_CHIP_P5:
                    // Parrot5 fifo configuration
                    // set RX fifo threshold to 16 bytes
                    // this will raise an IRQ when 16 empty slots are left
                    lcr |= UART_LCR_RXFIFO|UART_LCR_FIFO_THRESHOLD_16;
                    // TX fifo has no threshold on P5
                    lcr |= UART_LCR_TXFIFO;
                    break;

                case PARROT_CHIP_P5P:
                case PARROT_CHIP_P6:
                case PARROT_CHIP_P6I:
                    // Parrot5+ fifo configuration
                    // set RX fifo threshold to 48 bytes
                    // this will raise an IRQ when 16 empty slots are left
                    lcr |= UART_LCR_RXFIFO|UART_LCR_FIFO_RX_THRESHOLD_48;
                    // set TX fifo threshold to N bytes
                    // this will raise an IRQ when N chars are left to be sent
                    lcr |= UART_LCR_TXFIFO|LCR_TX_FIFO_THRESHOLD;
                    break;

                case PARROT_CHIP_P7:
                    break;

                case TI_CHIP_OMAP3630:
                case CPU_TYPES_UNKNOWN:
                    break;
            }
        }
        // hardware flow control configuration
        if ( new_cfg->flags & CYGNUM_SERIAL_FLOW_RTSCTS_TX ) {
            lcr |= UART_LCR_CTSMODE;
        }
        if ( new_cfg->flags & CYGNUM_SERIAL_FLOW_RTSCTS_RX ) {
            // do nothing, RTS is always generated
        }

        // write configuration
        HAL_WRITE_UINT32( base+_UART_LCR, lcr );

        // enable RX and TX interrupts
        HAL_WRITE_UINT32( base+_UART_IRX, UART_IRX_RIM );
        HAL_WRITE_UINT32( base+_UART_ITX, UART_ITX_TIM );

        // enable UART
        HAL_WRITE_UINT32( base+_UART_SPD, baud_divisor );
    }

    if ( new_cfg != &chan->config ) {
        chan->config = *new_cfg;
    }
    return true;
}

//==========================================================================
// Function to initialize the device.  Called at bootstrap time.

static bool
parrot5_serial_init(struct cyg_devtab_entry *tab) {
    serial_channel *chan = (serial_channel *)tab->priv;
    parrot5_serial_info *p5_chan = (parrot5_serial_info *)chan->dev_priv;

    // Really only required for interrupt driven devices
    (chan->callbacks->serial_init)(chan);

    if ( chan->out_cbuf.len != 0 ) {
        cyg_drv_interrupt_create( p5_chan->int_num,
                                  CYG_IO_SERIAL_PARROT5_INT_PRIORITY,
                                  (cyg_addrword_t)chan,
                                  parrot5_serial_isr,
                                  parrot5_serial_dsr,
                                  &p5_chan->serial_interrupt_handle,
                                  &p5_chan->serial_interrupt);
        cyg_drv_interrupt_attach(p5_chan->serial_interrupt_handle);
        cyg_drv_interrupt_unmask(p5_chan->int_num);
    }
    serial_config_port(chan, &chan->config, true);
    return true;
}

//==========================================================================
// This routine is called when the device is "looked" up (i.e. attached)

static Cyg_ErrNo
parrot5_serial_lookup( struct cyg_devtab_entry **tab,
                       struct cyg_devtab_entry *sub_tab,
                       const char *name ) {
    serial_channel *chan = (serial_channel *)(*tab)->priv;
    // Really only required for interrupt driven devices
    (chan->callbacks->serial_init)(chan);
    return ENOERR;
}

//==========================================================================
// Send a character to the device output buffer.
// Return 'true' if character is sent to device

static bool
parrot5_serial_putc( serial_channel *chan, unsigned char c ) {
    parrot5_serial_info *p5_chan = (parrot5_serial_info *)chan->dev_priv;
    CYG_ADDRWORD base = p5_chan->base;
    bool ret = false;
    cyg_uint32 status;

    HAL_READ_UINT32( base+_UART_STATUS, status );

    // UARTs have no TX full flag, so we need to implement
    // our own fill counter
    if (!( status & UART_STATUS_TXFILLED )) {
        // tx fifo is empty, reload fill counter
            switch (get_cpu_types())
            {
                case PARROT_CHIP_P5:
                    p5_chan->txfill = UART_TX_FIFO_SIZE_P5;
                    break;

                case PARROT_CHIP_P5P:
                case PARROT_CHIP_P6:
                case PARROT_CHIP_P6I:
                    p5_chan->txfill = UART_TX_FIFO_SIZE_P5P;
                    break;

                case PARROT_CHIP_P7:
                    p5_chan->txfill = UART_TX_FIFO_SIZE_P7;
                    break;

                case TI_CHIP_OMAP3630:
                case CPU_TYPES_UNKNOWN:
                    break;
            }
    }
    // Parrot5+ and successor has a configurable threshold, so IRQs may be raised when
    // fifo is not empty yet
    else if ( status & UART_STATUS_ITTX ) {

            // reload fill counter
            switch (get_cpu_types())
            {
                case PARROT_CHIP_P5:
                    break;

                case PARROT_CHIP_P6:
                case PARROT_CHIP_P6I:
                case PARROT_CHIP_P5P:
                    if (p5_chan->txfill < (UART_TX_FIFO_SIZE_P5P - TX_FIFO_THRESHOLD))
                        p5_chan->txfill = UART_TX_FIFO_SIZE_P5P - TX_FIFO_THRESHOLD;
                    break;

                case PARROT_CHIP_P7:
                    if (p5_chan->txfill < (UART_TX_FIFO_SIZE_P7 - TX_FIFO_THRESHOLD))
                        p5_chan->txfill = UART_TX_FIFO_SIZE_P7 - TX_FIFO_THRESHOLD;
                    break;

                case TI_CHIP_OMAP3630:
                case CPU_TYPES_UNKNOWN:
                    break;
            }
    }

    if ( p5_chan->txfill ) {
        HAL_WRITE_UINT32( base+_UART_TRX, c );
        p5_chan->txfill--;
        ret = true;
    }
    return ret;
}

//==========================================================================
// Fetch a character from the device input buffer, waiting if necessary

static unsigned char
parrot5_serial_getc(serial_channel *chan) {
    parrot5_serial_info *p5_chan = (parrot5_serial_info *)chan->dev_priv;
    CYG_ADDRWORD base = p5_chan->base;
    cyg_uint32 status;
    unsigned int c;

    // wait for a character
    while (1) {
        HAL_READ_UINT32( base+_UART_STATUS, status );

        if ( status & UART_STATUS_RXFILLED ) {
            HAL_READ_UINT32( base+_UART_TRX, c );
            // check for errors
            if (!( c & ~0xff )) {
                break;
            }
        }
    }
    return c;
}

//==========================================================================
// Set up the device characteristics; baud rate, etc.

static Cyg_ErrNo
parrot5_serial_set_config(serial_channel *chan, cyg_uint32 key,
                          const void *xbuf, cyg_uint32 *len) {
    switch ( key ) {

    case CYG_IO_SET_CONFIG_SERIAL_INFO: {
        cyg_serial_info_t *config = (cyg_serial_info_t *)xbuf;
        if ( *len < sizeof(cyg_serial_info_t) ) {
            return -EINVAL;
        }
        *len = sizeof(cyg_serial_info_t);
        if ( true != serial_config_port(chan, config, false) ) {
            return -EINVAL;
        }
    }
        break;

#ifdef CYGOPT_IO_SERIAL_FLOW_CONTROL_HW

    case CYG_IO_SET_CONFIG_SERIAL_HW_RX_FLOW_THROTTLE: {
        parrot5_serial_info *p5_chan = (parrot5_serial_info *)chan->dev_priv;
        CYG_ADDRWORD base = p5_chan->base;
        bool throttle = *(int *)xbuf;
        if ( *len != sizeof(int) ) {
            return -EINVAL;
        }


        switch (get_cpu_types())
        {
            case PARROT_CHIP_P5:
                // Parrot5 does not support manual control of nRTS pin, so we
                // mask (throttle)/unmask(unthrottle) RX irqs hoping automatic RTS
                // control will do the job for us...
                HAL_WRITE_UINT32( base+_UART_IRX, (throttle)? 0 : 1);
                break;

            case PARROT_CHIP_P5P:
            case PARROT_CHIP_P6:
            case PARROT_CHIP_P6I:
            case PARROT_CHIP_P7:
            {// Parrot5+ allows manual control of nRTS pin
                cyg_uint32 reg;
                HAL_READ_UINT32( base+_UART_LCR, reg );
                if ( throttle ) {
                    // force nRTS to 1
                    reg |= UART_LCR_RTS_CTRL;
                }
                else {
                    // restore automatic nRTS control
                    reg &= ~UART_LCR_RTS_CTRL;
                }
                HAL_WRITE_UINT32( base+_UART_LCR, reg );
            }
                break;

            case TI_CHIP_OMAP3630:
            case CPU_TYPES_UNKNOWN:
                break;
        }
    }
        break;

    case CYG_IO_SET_CONFIG_SERIAL_HW_FLOW_CONFIG:
        // Clear any unsupported flags here
        // DSR/DTR not supported
        if ( chan->config.flags &
             (CYGNUM_SERIAL_FLOW_DSRDTR_RX|CYGNUM_SERIAL_FLOW_DSRDTR_TX)) {
            chan->config.flags &= ~(CYGNUM_SERIAL_FLOW_DSRDTR_RX|
                                    CYGNUM_SERIAL_FLOW_DSRDTR_TX);
            return -EINVAL;
        }
        break;

#endif

    case CYG_IO_SET_CONFIG_SERIAL_HW_BREAK:
        {
            cyg_uint32 status;
	    cyg_int32 timeout = 5*1000;  // Wait 5 seconds max
	    cyg_serial_info_t *config = (cyg_serial_info_t *)xbuf;
	    parrot5_serial_info *p5_chan = (parrot5_serial_info *)chan->dev_priv;
	    CYG_ADDRWORD base = p5_chan->base;

	    if ( config->flags & CYGNUM_SERIAL_SET_BREAK ) {
		HAL_WRITE_UINT32(base+_UART_SETBRK, 0x1);

		while(timeout-- > 0)
		{
	            HAL_READ_UINT32( base+_UART_STATUS, status );
		    if ( !(status & UART_STATUS_STOP_BREAK) )
			break;
		}

		if ( timeout <= 0 )
		    return -EINVAL;

	    } else if ( config->flags & CYGNUM_SERIAL_RESET_BREAK ) {
		HAL_WRITE_UINT32(base+_UART_RSTBRK, 0x1);
	    } else
		return -EINVAL;

	    break;
	}

    default:
        return -EINVAL;
    }

    return ENOERR;
}

//==========================================================================
// Enable the transmitter on the device

static void
parrot5_serial_start_xmit(serial_channel *chan) {
    int mask;
    cyg_uint32 old;
    parrot5_serial_info *p5_chan = (parrot5_serial_info *)chan->dev_priv;
    CYG_ADDRWORD base = p5_chan->base;

    // enable TX interrupts
    HAL_WRITE_UINT32( base+_UART_ITX, UART_ITX_TIM );

    // we are going to write characters to the HW fifo, which may potentially
    // acknowledge TX interrupts when they have just been raised (and cause
    // spurious IRQs); to avoid this we mask UART IRQs during fifo
    // manipulations (which are done with the scheduler locked)
    HAL_DISABLE_INTERRUPTS(old);
    // get UART IRQ mask
    mask = hal_interrupt_get_mask(p5_chan->int_num);
    if ( mask ) {
        HAL_INTERRUPT_MASK(p5_chan->int_num);
    }
    HAL_RESTORE_INTERRUPTS(old);

    (chan->callbacks->xmt_char)(chan);

    // unmask UART IRQs
    HAL_DISABLE_INTERRUPTS(old);
    if ( mask ) {
        HAL_INTERRUPT_UNMASK(p5_chan->int_num);
    }
    HAL_RESTORE_INTERRUPTS(old);
}

//==========================================================================
// Disable the transmitter on the device

static void
parrot5_serial_stop_xmit(serial_channel *chan) {
    parrot5_serial_info *p5_chan = (parrot5_serial_info *)chan->dev_priv;
    CYG_ADDRWORD base = p5_chan->base;

    // disable TX interrupts
    HAL_WRITE_UINT32( base+_UART_RSTBRK, 0 );
}

//==========================================================================
// Serial I/O - low level interrupt handler (ISR)

static cyg_uint32
parrot5_serial_isr(cyg_vector_t vector, cyg_addrword_t data) {
    serial_channel *chan = (serial_channel *)data;
    parrot5_serial_info *p5_chan = (parrot5_serial_info *)chan->dev_priv;
    cyg_drv_interrupt_mask(p5_chan->int_num);
    cyg_drv_interrupt_acknowledge(p5_chan->int_num);
    return CYG_ISR_CALL_DSR;  // Cause DSR to be run
}

//==========================================================================
// Serial I/O - high level interrupt handler (DSR)

static void
parrot5_serial_dsr( cyg_vector_t vector, cyg_ucount32 count,
                    cyg_addrword_t data) {
    serial_channel *chan = (serial_channel *)data;
    parrot5_serial_info *p5_chan = (parrot5_serial_info *)chan->dev_priv;
    CYG_ADDRWORD base = p5_chan->base;
    cyg_uint32 status, c, strx;

    HAL_READ_UINT32( base+_UART_STATUS, status );

    // empty RX FIFO
    if ( status & UART_STATUS_ITRX ) {

        strx = status;

        while ( strx & UART_STATUS_RXFILLED ) {

            HAL_READ_UINT32( base+_UART_TRX, c );

            if ( c & ~0xff ) {
                // an error was detected
#ifdef CYGOPT_IO_SERIAL_SUPPORT_LINE_STATUS
                cyg_serial_line_status_t stat;
                stat.value = 1;
                if ( c & UART_TRX_OVERRUN ) {
                    stat.which = CYGNUM_SERIAL_STATUS_OVERRUNERR;
                    (chan->callbacks->indicate_status)(chan, &stat );
                }
                if ( c & UART_TRX_PARITY_ERROR ) {
                    stat.which = CYGNUM_SERIAL_STATUS_PARITYERR;
                    (chan->callbacks->indicate_status)(chan, &stat );
                }
                if ( c & UART_TRX_FRAME_ERROR ) {
                    stat.which = CYGNUM_SERIAL_STATUS_FRAMEERR;
                    (chan->callbacks->indicate_status)(chan, &stat );
                }
                if ( c & UART_TRX_RXBREAK ) {
                    stat.which = CYGNUM_SERIAL_STATUS_BREAK;
                    (chan->callbacks->indicate_status)(chan, &stat );
                }
#endif
            }
            else {
                (chan->callbacks->rcv_char)(chan, c );
            }

            HAL_READ_UINT32( base+_UART_STATUS, strx );
        }
    }

    // fill in TX FIFO
    if ( status & UART_STATUS_ITTX ) {
        (chan->callbacks->xmt_char)(chan);
    }

    cyg_drv_interrupt_unmask(p5_chan->int_num);
}

#endif // CYGDAT_IO_SERIAL_PARROT5_INL
