/**
 *
 *       @file  spi_parrot6i.h
 *
 *      @brief  
 *
 *     @author  Gregor Boirie <gregor.boirie@parrot.com>
 *       @date  07-May-2010
 *
 *        $Id: spi_parrot6i.h,v 1.3 2010-06-07 10:57:58 gboirie Exp $
 *
 *       Copyright (C) 2008 Parrot S.A.
 */

#ifndef _SPI_PARROT6I_H
#define _SPI_PARROT6I_H

#include <pkgconf/system.h>
#include <cyg/hal/hal_tables.h>
#ifdef CYGPKG_IO_SPI
#include <cyg/io/spi.h>
#endif

#define P6I_SPI_BUS_ENTRY CYG_HAL_TABLE_ENTRY(p6i_spi_buses)

/**
 * P6i SPI bus descriptor
 */
struct p6i_spi_bus {
#ifdef CYGPKG_IO_SPI
    struct cyg_spi_bus      _super;
#endif
    CYG_ADDRESS             base;   /** Bus base address */
    struct pclock const*    clock;  /** Bus clock */
} CYG_HAL_TABLE_TYPE;

/**
 * P6I SPI device descriptor
 */
struct p6i_spi_dev {
#ifdef CYGPKG_IO_SPI
    struct cyg_spi_device       _super;
#else
    struct p6i_spi_bus const*   spi_bus;    /** Bus this device is on */
#endif
    unsigned                    init:1;     /** Set to 1 when device is initialized */
    unsigned                    cs:1;       /** Set to 1 when device is selected */
    unsigned                    tsetupcs:4; /** Slave select setup time */
    unsigned                    tholdcs:4;  /** Slave select holding time */
    unsigned                    cpol:1;     /** Clock polarity */
    unsigned                    cpha:1;     /** Clock phase */
    unsigned                    lsb:1;      /** Least significant bit first */
    unsigned int                hz;         /** Bus clock speed in Hz */
} CYG_HAL_TABLE_TYPE;

/**
 * Define a P6i SPI bus
 */
#define P6I_DEFINE_SPI_BUS(_name, _base)            \
    struct p6i_spi_bus _name P6I_SPI_BUS_ENTRY = {  \
        .base = _base                               \
    }

/**
 * Define a P6i SPI device
 */
#ifdef CYGPKG_IO_SPI
#define P6I_DEFINE_SPI_DEV(_name, _bus, _setupcs, _holdcs, _cpol, _cpha, _lsb, _hz) \
    struct p6i_spi_dev _name = {                                                    \
        ._super = {                                                                 \
            .spi_bus    = (struct cyg_spi_bus*) _bus                                \
        },                                                                          \
        .init       = 0,                                                            \
        .cs         = 0,                                                            \
        .tsetupcs   = ((_hz * _setupcs) / 1000000000) - 2,                          \
        .tholdcs    = ((_hz * _holdcs) / 1000000000) - 2,                           \
        .cpol       = !! _cpol,                                                     \
        .cpha       = !! _cpha,                                                     \
        .lsb        = !! _lsb,                                                      \
        .hz         = _hz                                                           \
    }

#define p6i_spi_dev_bus(_device)                    \
    ({                                              \
        struct p6i_spi_dev* const _dev = _device;   \
        ((struct cyg_spi_device*) _dev)->spi_bus;   \
     })

#else   /* ! CYGPKG_IO_SPI */
#define P6I_DEFINE_SPI_DEV(_name, _bus, _setupcs, _holdcs, _cpol, _cpha, _lsb, _hz) \
    struct p6i_spi_dev _name = {                                                    \
        .spi_bus    = _bus,                                                         \
        .init       = 0,                                                            \
        .cs         = 0,                                                            \
        .tsetupcs   = ((_hz * _setupcs) / 1000000000) - 2,                          \
        .tholdcs    = ((_hz * _holdcs) / 1000000000) - 2,                           \
        .cpol       = !! _cpol,                                                     \
        .cpha       = !! _cpha,                                                     \
        .lsb        = !! _lsb,                                                      \
        .hz         = _hz                                                           \
    }

#define p6i_spi_dev_bus(_device)                    \
    ({                                              \
        struct p6i_spi_dev* const _dev = _device;   \
        _dev->spi_bus;                              \
     })

#endif  /* CYGPKG_IO_SPI */

extern void p6i_begin_spi(struct p6i_spi_dev*);
extern void p6i_end_spi(struct p6i_spi_dev*);
extern void p6i_set_spi_speed(struct p6i_spi_bus const*, unsigned int);
extern void p6i_xfer_spi_dev(struct p6i_spi_dev*,
                             cyg_bool,
                             cyg_uint32,
                             cyg_uint8 const*,
                             cyg_uint8*,
                             cyg_bool);

#endif

/* vim:set ts=4:sw=4:et:ft=c: */
