/**
 * @file   lpo.h
 * @brief  Parrot5(+) LPO support routines
 *
 * Parrot5 LPO block programming. Note: Emergency (PSA) mode is not supported
 * yet.
 *
 * @author ivan.djelic@parrot.com
 * @date   2008-03-27
 *
 * This file is licensed under the eCos Public License. A copy of this license
 * can be obtained from:
 * http://ecos.sourceware.org/license-overview.html
 *
 * The Original Code is eCos - Embedded Configurable Operating System,
 *
 * Portions created by Parrot S.A. are
 * Copyright (C) 2004-2008 Parrot S.A.
 */

#ifndef CYGONCE_LPO_H
#define CYGONCE_LPO_H

#include <cyg/infra/cyg_type.h>
#include <cyg/error/codes.h>
#include <cyg/kernel/kapi.h>

#define LPO_READ(_reg, _val) HAL_READ_UINT32( PARROT5_LPO+(_reg), _val );

#define LPO_WRITE(_reg, _val) {                                 \
        cyg_uint32 __reg;                                       \
        do {                                                    \
            HAL_READ_UINT32( PARROT5_LPO+_LPO_CONTROL, __reg ); \
        } while ( __reg & LPO_CTRL_WRITE_STAT );                \
        HAL_WRITE_UINT32( PARROT5_LPO+(_reg), _val );           \
    }

#define LPO_EVENT_ALL        (LPO_EVENT_RTCA|LPO_EVENT_EMCY|LPO_EVENT_WKUP)


/* Power management */

void parrot5_lpo_stop(void);
Cyg_ErrNo parrot5_lpo_standby(cyg_uint32 rtc_alarm,
                              int wakeup_enable,
                              int wakeup_level,
                              int emergency_enable,
                              int emergency_level,
                              cyg_uint8 scratch);
cyg_uint8 parrot5_lpo_get_scratch(void);

void lpo_standby_enable(cyg_uint32 rtc_alarm);
void parrot5_lpo_standby_lock(void);
void parrot5_lpo_standby_unlock(void);

/* Event handling */

#define LPO_EVENT_RTCA       (1 << 0)
#define LPO_EVENT_EMCY       (1 << 1)
#define LPO_EVENT_WKUP       (1 << 2)

typedef void (* lpo_callback_t)(int event, cyg_addrword_t data);

Cyg_ErrNo parrot5_lpo_event_register(int event, int pin_lvl,
                                     lpo_callback_t cb,
                                     cyg_addrword_t data,
                                     cyg_uint32 rtc_alarm);
void parrot5_lpo_event_unregister(int event);

#endif /* CYGONCE_LPO_H */
