//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * @file   parrot5.cxx
 * @brief  Wallclock implementation for Parrot5/5+
 *
 * @author ivan.djelic@parrot.com
 * @date   2008-03-27
 *
 * This file is licensed under the eCos Public License. A copy of this license
 * can be obtained from:
 * http://ecos.sourceware.org/license-overview.html
 *
 * The Original Code is eCos - Embedded Configurable Operating System,
 *
 * Portions created by Parrot S.A. are
 * Copyright (C) 2004-2008 Parrot S.A.
 */

#include <pkgconf/hal.h>                // Platform specific configury
#include <pkgconf/wallclock.h>          // Wallclock device config
#include <pkgconf/devices_wallclock_parrot5.h>

#include <cyg/hal/hal_io.h>             // IO macros
#include <cyg/hal/hal_intr.h>           // interrupt enable/disable
#include <cyg/infra/cyg_type.h>         // Common type definitions and support
#include <string.h>                     // memcpy()
#include <cyg/io/wallclock.hxx>         // The WallClock API

#include <cyg/hal/p5dev.h>              // platform definitions

#include "parrot5_lpo.h"

#define LPO_READ(_reg, _val) HAL_READ_UINT32(PARROT5_LPO+(_reg), _val);

#define LPO_WRITE(_reg, _val) {                                 \
        cyg_uint32 __reg;                                       \
        do {                                                    \
            HAL_READ_UINT32(PARROT5_LPO+_LPO_CONTROL, __reg);   \
        } while (__reg & LPO_CTRL_WRITE_STAT);                  \
        HAL_WRITE_UINT32(PARROT5_LPO+(_reg), _val);             \
    }

// Returns the number of seconds elapsed since 1970-01-01 00:00:00.
cyg_uint32 Cyg_WallClock::get_hw_seconds(void)
{
    cyg_uint32 now1, now2;

    // due to asynchronous clock domains, we must read this value twice
    do {
        LPO_READ(_LPO_RTCCNT, now1);
        LPO_READ(_LPO_RTCCNT, now2);
    } while (now1 != now2);

    // This will wrap around year 2106 :-)
    return now1;
}

#ifdef CYGSEM_WALLCLOCK_SET_GET_MODE

// Sets the clock. Argument is seconds elapsed since 1970-01-01 00:00:00.
void Cyg_WallClock::set_hw_seconds(cyg_uint32 secs)
{
    LPO_WRITE(_LPO_RTCCNT, secs);
}

#endif

// External function used to check if the 32kHz source is external or not
extern "C" int CYGDAT_DEVICES_WALLCLOCK_PARROT5_IS_32K_EXTERNAL_CHECK(void);

void Cyg_WallClock::init_hw_seconds(void)
{
	if (((CYGDAT_WALLCLOCK_PARROT5_32K_SOURCE_MODE == 3) &&             \
         (CYGDAT_DEVICES_WALLCLOCK_PARROT5_IS_32K_EXTERNAL_CHECK() == 0)) || \
        (CYGDAT_WALLCLOCK_PARROT5_32K_SOURCE_MODE == 2)) {
		cyg_uint32 reg;

	    // Use internal clock generation (see Parrot5 User's Manual 6.3.3)
	    LPO_READ(_LPO_CONTROL, reg);
	    LPO_WRITE(_LPO_CONTROL, reg|LPO_CTRL_32K_MOD);
	}

#ifndef CYGSEM_WALLCLOCK_SET_GET_MODE
	// initialise to 1970-01-01 00:00:00
    LPO_WRITE(_LPO_RTCCNT, 0);
#endif
}

//-----------------------------------------------------------------------------
// End of parrot5.cxx
