// #========================================================================
// #
// #    arm9.S
// #
// #    ARM exception vectors
// #
// #========================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003 Red Hat, Inc.
// Copyright (C) 2004 eCosCentric Ltd.
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
// #========================================================================
// ######DESCRIPTIONBEGIN####
// #
// # Author(s):     nickg
// # Contributors:  nickg
// # Date:          2004-04-21
// # Purpose:       ARM9 assembler functions
// # Description:
// #
// #####DESCRIPTIONEND####
// #
// #========================================================================

#include <pkgconf/hal.h>
#include <pkgconf/hal_arm.h>
#include <pkgconf/hal_arm_arm9.h>

//==========================================================================

#include <cyg/hal/arch.inc>

// Function definition, start executing body in ARM mode
#define FUNC_START_ARM(_name_, _r_) \
        .globl _name_; \
_name_:

#define PTR(name)               \
.##name: .word  name

//==========================================================================

#if !(defined(CYGSEM_HAL_ARM_ARM9_MMH_MMU) || \
      defined(CYGSEM_HAL_ARM_ARM9_MMH_PU)  || \
      defined(CYGSEM_HAL_ARM_ARM9_MMH_NONE) )
#error No Memory Management Hardware defined
#endif

//==========================================================================
// Set the TTB and DACR MMU registers

#if defined(CYGSEM_HAL_ARM_ARM9_MMH_MMU)
FUNC_START_ARM(cyg_hal_arm9_set_mmuregs,r2)
        mcr     p15,0,r0,c2,c0,0        // set TT Base
        mcr     p15,0,r1,c3,c0,0        // Set DACR
        bx      lr
#endif

//==========================================================================

FUNC_START_ARM(cyg_hal_arm9_soft_reset,r1)

    /* It would probably make more sense to have the
       clear/drain/invalidate after disabling the cache and MMU, but
       then we'd have to know the (unmapped) address of this code. */

        mrs     r1,cpsr
        bic     r1,r1,#0x1F       /* Put processor in SVC mode */
        orr     r1,r1,#0x13
        msr     cpsr,r1

        mov     r1, #0
        mcr     p15,0,r1,c7,c7,0  /* clear I+DCache */
        mcr     p15,0,r1,c7,c10,4 /* Drain Write Buffer */
#if defined(CYGSEM_HAL_ARM_ARM9_MMH_MMU)
        mcr     p15,0,r1,c8,c7,0  /* Invalidate TLBs */
        mrc     p15,0,r1,c1,c0,0
#endif
        bic     r1,r1,#0x1000     /* disable ICache */
        bic     r1,r1,#0x0007     /* disable DCache, MMU and alignment faults */
        mcr     p15,0,r1,c1,c0,0
        nop                       /* delay 1 */
        mov     pc, r0            /* delay 2  - next instruction should
                                    be fetched flat */

1:
        b       1b                /* Loop forever */

        bx      lr


//==========================================================================
// Instruction cache ops

// Enable the instruction cache
FUNC_START_ARM(hal_icache_enable,r1)
        mrc  p15,0,r1,c1,c0,0
        orr  r1,r1,#0x1000
        mcr  p15,0,r1,c1,c0,0
        bx      lr

// Disable the instruction cache (and invalidate it, required semanitcs)
FUNC_START_ARM(hal_icache_disable,r1)
        mrc    p15,0,r1,c1,c0,0
        bic    r1,r1,#0x1000    /* Disable ICache */
        mcr    p15,0,r1,c1,c0,0
        mov    r1,#0
        mcr    p15,0,r1,c7,c5,0 /* flush ICache */
        nop                     /* next few instructions may be via cache */
        nop
        nop
        nop
        nop
        nop
        bx      lr

// Query the state of the instruction cache
FUNC_START_ARM(hal_icache_is_enabled,r0)
        mrc     p15,0,r0,c1,c0,0
        bx      lr

// Invalidate the entire cache
FUNC_START_ARM(hal_icache_invalidate_all,r1)
    /* this function can discard dirty cache lines (N/A for ICache) */
        mov    r1,#0
        mcr    p15,0,r1,c7,c5,0         /* flush ICache */
#if defined(CYGSEM_HAL_ARM_ARM9_MMH_MMU)
        mcr    p15,0,r1,c8,c5,0         /* flush ITLB only */
#endif
        nop                             /* next few instructions may be
                                            via cache */
        nop
        nop
        nop
        nop
        nop
        bx      lr

//==========================================================================
// Data cache ops

// Enable the data cache
FUNC_START_ARM(hal_dcache_enable,r1)
        push    {lr}
        bl      hal_dcache_invalidate_all
        mrc     p15,0,r1,c1,c0,0
        orr     r1,r1,#0xC          /* enable DCache and write buffer */
        mcr     p15,0,r1,c1,c0,0
        pop     {lr}
        bx      lr

// Disable the data cache (and invalidate it, required semanitcs)
FUNC_START_ARM(hal_dcache_disable,r1)
        push    {lr}
        bl      hal_dcache_sync_clean
        mrc     p15,0,r1,c1,c0,0
        bic     r1,r1,#0xC          /* disable DCache AND write buffer  */
        mcr     p15,0,r1,c1,c0,0
        mov     r1,#0
        mcr     p15,0,r1,c7,c6,0    /* clear data cache */
        pop     {lr}
        bx      lr

// Query the state of the data cache
FUNC_START_ARM(hal_dcache_is_enabled,r0)
        mrc     p15,0,r0,c1,c0,0
        bx      lr

// Flush the entire dcache (and then both TLBs, just in case)
FUNC_START_ARM(hal_dcache_invalidate_all,r0)
        /* this function can discard dirty cache lines. */
        mov     r0,#0
        mcr     p15,0,r0,c7,c6,0            /* invalidate d-cache */
#if defined(CYGSEM_HAL_ARM_ARM9_MMH_MMU)
        mcr     p15,0,r0,c8,c7,0            /* flush i+d-TLBs */
#endif
        bx      lr

// Synchronize the contents of the cache with memory.
FUNC_START_ARM(hal_dcache_sync_clean,r0)
#if defined(CYGPKG_HAL_ARM_ARM9_ARM926E)
1:      mrc     p15,0,r15,c7,c14,3
        bne	    1b
#elif defined(CYGPKG_HAL_ARM_ARM9_ARM920T)
        mov	    r1,#7<<5
1:
        orr	    r3,r1,#63<<26
2:
        mcr	    p15,0,r3,c7,c14,2
        subs    r3,r3,#1<<26
        bcs     2b
        subs    r1,r1,#1<<5
        bcs     1b
#else
#error Don't know how to flush cache !
#endif
        mov     r0,#0
        mcr     p15,0,r0,c7,c10,4        /* and drain the write buffer */
        bx      lr

// --------------------------------------------------------------------------
//  end of arm9.S

