//=============================================================================
//
// MLT linker script for ARM
//
// modified to support memory protection
//
//=============================================================================

#include <pkgconf/system.h>
#if defined(CYGPKG_BASEBAND) || defined(CYGPKG_PARROT5_BASEBAND)
#  define LIBTARGET libtarget_mp.a libbaseband_mp.a
#else
#  define LIBTARGET libtarget_mp.a
#endif

STARTUP(vectors.o)
ENTRY(reset_vector)
// use redirected version of object code
INPUT(extras_mp.o)
INPUT(memprot_mp.o)
#if (__GNUC__ >= 3)
GROUP(LIBTARGET libgcc.a libsupc++.a)
#else
GROUP(LIBTARGET libgcc.a)
#endif

#if defined(__ARMEB__)
OUTPUT_FORMAT(elf32-bigarm)
#endif

// Keep RODATA in separate sections.
#define MERGE_IN_RODATA
#undef  CYGPRI_PID_BE_WORKAROUND

#define ALIGN_LMA 4
#define FOLLOWING(_section_) AT ((LOADADDR (_section_) + SIZEOF (_section_) + ALIGN_LMA - 1) & ~ (ALIGN_LMA - 1))
#define LMA_EQ_VMA
#define FORCE_OUTPUT . = .

// Some versions of gcc define "arm" which causes problems with .note.arm.ident
#undef arm
#define SECTIONS_BEGIN                          \
  /* Debug information */                       \
  .debug_aranges  0 : { *(.debug_aranges) }     \
  .debug_pubnames 0 : { *(.debug_pubnames) }    \
  .debug_info     0 : { *(.debug_info) }        \
  .debug_abbrev   0 : { *(.debug_abbrev) }      \
  .debug_line     0 : { *(.debug_line) }        \
  .debug_frame    0 : { *(.debug_frame) }       \
  .debug_str      0 : { *(.debug_str) }         \
  .debug_loc      0 : { *(.debug_loc) }         \
  .debug_macinfo  0 : { *(.debug_macinfo) }     \
  .note.arm.ident 0 : { KEEP (*(.note.arm.ident)) }

#define SECTION_fixed_vectors(_region_, _vma_, _lma_) \
    .fixed_vectors _vma_ : _lma_ \
    { FORCE_OUTPUT; KEEP (*(.fixed_vectors)) } \
    > _region_

#define SECTION_rom_vectors(_region_, _vma_, _lma_) \
    .rom_vectors _vma_ : _lma_ \
    { __rom_vectors_vma = ABSOLUTE(.); \
      FORCE_OUTPUT; KEEP (*(.vectors)) } \
    > _region_ \
    __rom_vectors_lma = LOADADDR(.rom_vectors);

#define SECTION_text(_region_, _vma_, _lma_) \
    .text _vma_ : _lma_ \
    { _stext = ABSOLUTE(.); \
    PROVIDE (__stext = ABSOLUTE(.)); \
    *(.text*) *(.gnu.warning) *(.gnu.linkonce.t.*) *(.init) \
    *(.glue_7) *(.glue_7t)  \
    } > _region_ \
    _etext = .; PROVIDE (__etext = .);

#define SECTION_fini(_region_, _vma_, _lma_) \
    .fini _vma_ : _lma_ \
    { FORCE_OUTPUT; *(.fini) } \
    > _region_

#if defined(CYGPRI_PID_BE_WORKAROUND)
#define SECTION_rodata(_region_, _vma_, _lma_)
#define SECTION_rodata1(_region_, _vma_, _lma_)
#else
#define SECTION_rodata(_region_, _vma_, _lma_) \
    .rodata _vma_ : _lma_ \
    { FORCE_OUTPUT; *(.rodata*) *(.gnu.linkonce.r.*) } \
    > _region_

#define SECTION_rodata1(_region_, _vma_, _lma_) \
    .rodata1 _vma_ : _lma_ \
    { FORCE_OUTPUT; *(.rodata1) } \
    > _region_
#endif // CYGPRI_PID_BE_WORKAROUND

#define SECTION_fixup(_region_, _vma_, _lma_) \
    .fixup _vma_ : _lma_ \
    { FORCE_OUTPUT; *(.fixup) } \
    > _region_

#define SECTION_gcc_except_table(_region_, _vma_, _lma_) \
    .gcc_except_table _vma_ : _lma_ \
    { FORCE_OUTPUT; *(.gcc_except_table) } \
    > _region_

#define SECTION_eh_frame(_region_, _vma_, _lma_)      \
  .eh_frame _vma_ : _lma_                             \
    {                                                 \
       FORCE_OUTPUT;  __EH_FRAME_BEGIN__ = .;         \
       KEEP(*(.eh_frame))                             \
       __FRAME_END__ = .;                             \
       . = . + 8;                                     \
    } > _region_ = 0

#define SECTION_RELOCS(_region_, _vma_, _lma_)                              \
  .rel.text      :                                                          \
    {                                                                       \
      *(.rel.text)                                                          \
      *(.rel.text.*)                                                        \
      *(.rel.gnu.linkonce.t*)                                               \
    } > _region_                                                            \
  .rela.text     :                                                          \
    {                                                                       \
      *(.rela.text)                                                         \
      *(.rela.text.*)                                                       \
      *(.rela.gnu.linkonce.t*)                                              \
    } > _region_                                                            \
  .rel.data      :                                                          \
    {                                                                       \
      *(.rel.data)                                                          \
      *(.rel.data.*)                                                        \
      *(.rel.gnu.linkonce.d*)                                               \
    } > _region_                                                            \
  .rela.data     :                                                          \
    {                                                                       \
      *(.rela.data)                                                         \
      *(.rela.data.*)                                                       \
      *(.rela.gnu.linkonce.d*)                                              \
    } > _region_                                                            \
  .rel.rodata    :                                                          \
    {                                                                       \
      *(.rel.rodata)                                                        \
      *(.rel.rodata.*)                                                      \
      *(.rel.gnu.linkonce.r*)                                               \
    } > _region_                                                            \
  .rela.rodata   :                                                          \
    {                                                                       \
      *(.rela.rodata)                                                       \
      *(.rela.rodata.*)                                                     \
      *(.rela.gnu.linkonce.r*)                                              \
    } > _region_                                                            \
  .rel.got       :   { *(.rel.got)    } > _region_                          \
  .rela.got      :   { *(.rela.got)   } > _region_                          \
  .rel.ctors     :   { *(.rel.ctors)  } > _region_                          \
  .rela.ctors    :   { *(.rela.ctors) } > _region_                          \
  .rel.dtors     :   { *(.rel.dtors)  } > _region_                          \
  .rela.dtors    :   { *(.rela.dtors) } > _region_                          \
  .rel.init      :   { *(.rel.init)   } > _region_                          \
  .rela.init     :   { *(.rela.init)  } > _region_                          \
  .rel.fini      :   { *(.rel.fini)   } > _region_                          \
  .rela.fini     :   { *(.rela.fini)  } > _region_                          \
  .rel.bss       :   { *(.rel.bss)    } > _region_                          \
  .rela.bss      :   { *(.rela.bss)   } > _region_                          \
  .rel.plt       :   { *(.rel.plt)    } > _region_                          \
  .rela.plt      :   { *(.rela.plt)   } > _region_                          \
  .rel.dyn       :   { *(.rel.dyn)    } > _region_

#define SECTION_got(_region_, _vma_, _lma_)                                 \
  .got _vma_ : _lma_                                                        \
    {                                                                       \
      FORCE_OUTPUT; *(.got.plt) *(.got)                                     \
      _GOT1_START_ = ABSOLUTE (.); *(.got1) _GOT1_END_ = ABSOLUTE (.);      \
      _GOT2_START_ = ABSOLUTE (.); *(.got2) _GOT2_END_ = ABSOLUTE (.);      \
    } > _region_

#define SECTION_mmu_tables(_region_, _vma_, _lma_) \
    .mmu_tables _vma_ : _lma_ \
    { FORCE_OUTPUT; *(.mmu_tables) } \
    > _region_

#define SECTION_sram(_region_, _vma_, _lma_) \
    .sram _vma_ : _lma_ \
    { FORCE_OUTPUT; *(.sram*) } \
    > _region_

#define SECTION_data(_region_,  _vma_, _lma_) \
    .data _vma_ : _lma_ \
    { __ram_data_start = ABSOLUTE (.); \
    *(.data*) *(.data1) *(.gnu.linkonce.d.*) MERGE_IN_RODATA \
    . = ALIGN (4); \
    KEEP(*( SORT (.ecos.table.*))) ;            \
    . = ALIGN (4); \
    __CTOR_LIST__ = ABSOLUTE (.); KEEP (*(SORT (.ctors*))) __CTOR_END__ = ABSOLUTE (.); \
    __DTOR_LIST__ = ABSOLUTE (.); KEEP (*(SORT (.dtors*))) __DTOR_END__ = ABSOLUTE (.); \
    *(.dynamic) *(.sdata*) *(.gnu.linkonce.s.*) \
    . = ALIGN (4); *(.2ram.*) } \
    > _region_ \
    __rom_data_start = LOADADDR (.data); \
    __ram_data_end = .; PROVIDE (__ram_data_end = .); _edata = .; PROVIDE (edata = .); \
    PROVIDE (__rom_data_end = LOADADDR (.data) + SIZEOF(.data));

#define SECTION_bss(_region_,  _vma_, _lma_) \
    .bss _vma_ : _lma_ \
    { __bss_start = ABSOLUTE (.); \
    *(.scommon) *(.dynsbss) *(.sbss*) *(.gnu.linkonce.sb.*) \
    *(.dynbss) *(.bss*) *(.gnu.linkonce.b.*) *(COMMON) \
    __bss_end = ABSOLUTE (.); } \
    > _region_

#define SECTIONS_END . = ALIGN(4); _end = .; PROVIDE (end = .);

/* Memory protection specific macros start here */

#include <cyg/infra/cyg_type.inc>

#include <pkgconf/memprot.h>
#include <pkgconf/system.h>
#include CYGHWR_MEMORY_LAYOUT_H

/* include MMU definitions */
#include <cyg/hal/hal_mmu.h>

// Maximum per process heap size defaults to 64 MB
#define HEAP_DEFAULT_SIZE    (64*MMU_SECTION_SIZE)
// Minimum per process heap size is 4 KB
#define HEAP_MINIMAL_SIZE    (MMU_PAGE_SIZE)
// Maximum per process stack area defaults to 16 MB
#define STACKS_DEFAULT_SIZE  (16*MMU_SECTION_SIZE)

// this is used to trick heapgen.tcl and avoid parsing '__heap1' twice.
#define CYG_LABEL_DEFN2(_label_) CYG_LABEL_DEFN(_label_)

// this macro is used to align an area on 4 KB boundaries
#define MMU_ALIGN(_tag_,_declaration_) \
    . = ALIGN (MMU_PAGE_SIZE); \
    __memprot_area_start_##_tag_ = ABSOLUTE(.); \
    _declaration_ \
    . = ALIGN (MMU_PAGE_SIZE); \
    __memprot_area_end_##_tag_ = ABSOLUTE(.);

// this macro is used to align an area on 1 MB boundaries
#define MMU_SECTION_ALIGN(_tag_,_declaration_) \
    . = ALIGN (MMU_SECTION_SIZE); \
    __memprot_area_start_##_tag_ = ABSOLUTE(.); \
    _declaration_ \
    . = ALIGN (MMU_SECTION_SIZE); \
    __memprot_area_end_##_tag_ = ABSOLUTE(.);

// this macro does not require any alignment
#define AREA(_tag_,_declaration_) \
    __memprot_area_start_##_tag_ = ABSOLUTE(.); \
    _declaration_ \
    __memprot_area_end_##_tag_ = ABSOLUTE(.);

// Virtual process text section
#define SECTION_vp_text(_region_, _vma_, _lma_, _n_) \
    .text##_n_ _vma_ : _lma_ { FORCE_OUTPUT; \
      *(__memprot_vp##_n_##_.text*) \
      *(__memprot_vp##_n_##_.gnu.warning) \
      *(__memprot_vp##_n_##_.gnu.linkonce.t.*) \
      *(__memprot_vp##_n_##_.init) \
      *(__memprot_vp##_n_##_.glue_7) \
      *(__memprot_vp##_n_##_.glue_7t)  \
    } > _region_

// Virtual process data section
#define SECTION_vp_data(_region_, _vma_, _lma_, _n_) \
    .data##_n_ _vma_ : _lma_ { FORCE_OUTPUT; \
     *(__memprot_vp##_n_##_.data*) \
     *(__memprot_vp##_n_##_.data1) \
     *(__memprot_vp##_n_##_.gnu.linkonce.d.*) MERGE_IN_RODATA \
    . = ALIGN (4); \
     *(__memprot_vp##_n_##_.dynamic) \
     *(__memprot_vp##_n_##_.sdata*) \
     *(__memprot_vp##_n_##_.gnu.linkonce.s.*) \
    . = ALIGN (4); \
     *(__memprot_vp##_n_##_.2ram.*) } > _region_ \
    __rom_data_start_##_n_ = LOADADDR (.data##_n_); \
    __rom_data_end_##_n_ = LOADADDR (.data##_n_) + SIZEOF(.data##_n_);

// Virtual process bss section
#define SECTION_vp_bss(_region_, _vma_, _lma_, _n_) \
    .bss##_n_ _vma_ : _lma_ { FORCE_OUTPUT; \
    *(__memprot_vp##_n_##_.scommon) \
    *(__memprot_vp##_n_##_.dynsbss) \
    *(__memprot_vp##_n_##_.sbss*) \
    *(__memprot_vp##_n_##_.gnu.linkonce.sb.*) \
    *(__memprot_vp##_n_##_.dynbss) \
    *(__memprot_vp##_n_##_.bss*) \
    *(__memprot_vp##_n_##_.gnu.linkonce.b.*) \
    } > _region_

// Virtual process read-only data section
#define SECTION_vp_rodata(_region_, _vma_, _lma_, _n_) \
    .rodata##_n_ _vma_ : _lma_ { FORCE_OUTPUT;\
    *(__memprot_vp##_n_##_.rodata*) \
    *(__memprot_vp##_n_##_.gnu.linkonce.r.*) \
    } > _region_

// Virtual process heap section
#define SECTION_vp_heap(_region_, _vma_, _lma_, _n_) \
    . = _vma_; \
    __heap_top_##_n_ = _vma_; \
    . += HEAP_DEFAULT_SIZE;

// Virtual process stacks section
#define SECTION_vp_stacks(_region_, _vma_, _lma_, _n_) \
    . = _vma_; \
    . += STACKS_DEFAULT_SIZE;

#define FOLLOW_MMU_ALIGNED(_section_) \
((LOADADDR(_section_)+SIZEOF(_section_)+MMU_PAGE_SIZE-1) & ~(MMU_PAGE_SIZE-1))

// Virtual process section (includes text, rodata, data, bss, heap and stacks)
#define VP(_region_, _vma_, _lma_, _n_ ) \
  . = ABSOLUTE(_vma_) ; \
  __rom_ro_start_##_n_ = _lma_; \
  MMU_ALIGN( ro_##_n_, \
    SECTION_vp_text(_region_, _vma_, AT _lma_, _n_) \
    SECTION_vp_rodata(_region_, ALIGN (0x4), FOLLOWING(.text##_n_), _n_)) \
  __rom_ro_end_##_n_ = FOLLOW_MMU_ALIGNED (.rodata##_n_); \
  MMU_ALIGN( rw_##_n_, \
        MMU_ALIGN(data_##_n_,SECTION_vp_data(_region_,ALIGN(0x4), \
            AT FOLLOW_MMU_ALIGNED(.rodata##_n_),_n_)) \
    MMU_ALIGN(bss_##_n_,SECTION_vp_bss(_region_,ALIGN(0x4),LMA_EQ_VMA, _n_))) \
  MMU_ALIGN(heap_##_n_,SECTION_vp_heap(_region_,ALIGN(0x4),LMA_EQ_VMA,_n_)) \
  MMU_ALIGN(stacks_##_n_,SECTION_vp_stacks(_region_,ALIGN(0x4),LMA_EQ_VMA,_n_))

#define SECTION_heap(_region_, _vma_, _lma_ ) \
    . = _vma_; \
    CYG_LABEL_DEFN(__heap1) = ALIGN (0x8); \
    . = CYG_LABEL_DEFN(__heap1) + CYGMEM_SECTION_heap1_SIZE; \
    CYG_LABEL_DEFN(__heap1_end) = .;

#define SECTION_bss_BaseBand(_region_, _vma_, _lma_ ) \
    .bss.BaseBand _vma_ : _lma_ { *(*.BaseBand) } > _region_

// Additional sections for Parrot5 ASIC
#ifdef CYGPKG_HAL_ARM_ARM9_P5DEV

#define SECTION_itcm(_region_, _vma_, _lma_) \
    .text.itcm _vma_ : _lma_ \
    { FORCE_OUTPUT;_stext_itcm = ABSOLUTE(.); \
    PROVIDE (__stext_itcm = ABSOLUTE(.)); \
    *(.text.itcm) *(.text.itcm) } > _region_ \
    __stext_itcm_rom = LOADADDR (.text.itcm); \
    _etext_itcm = .; PROVIDE (__etext_itcm = .);

#define SECTION_dtcm(_region_,  _vma_, _lma_) \
    .data.dtcm _vma_ : _lma_ \
    { FORCE_OUTPUT;__ram_data_dtcm_start = ABSOLUTE (.); \
    *(.data.dtcm) *(.data.dtcm) } > _region_ \
    __rom_data_dtcm_start = LOADADDR (.data.dtcm); \
    __ram_data_dtcm_end = .; PROVIDE (__ram_data_dtcm_end = .); _edata_dtcm = .; PROVIDE (edata_dtcm = .); \
    PROVIDE (__rom_data_dtcm_end = LOADADDR (.data.dtcm) + SIZEOF(.data.dtcm));

#define SECTION_text_fast(_region_, _vma_, _lma_) \
    .text.fast _vma_ : _lma_ \
    { FORCE_OUTPUT;_stext_fast = ABSOLUTE(.); \
    PROVIDE (__stext_fast = ABSOLUTE(.)); \
    *(.text.itcm) *(.text.fast) } > _region_ \
    __stext_fast_rom = LOADADDR (.text.fast); \
    _etext_fast = .; PROVIDE (__etext_fast = .);

#define SECTION_data_fast(_region_,  _vma_, _lma_) \
    .data.fast _vma_ : _lma_ \
    { FORCE_OUTPUT;__ram_data_fast_start = ABSOLUTE (.); \
    *(.data.dtcm) *(.data.fast) } > _region_ \
    __rom_data_fast_start = LOADADDR (.data.fast); \
    __ram_data_fast_end = .; PROVIDE (__ram_data_fast_end = .); _edata_fast = .; PROVIDE (edata_fast = .); \
    PROVIDE (__rom_data_fast_end = LOADADDR (.data.fast) + SIZEOF(.data.fast));

#endif

//----------------------------------------------------------------------------
// Memory layout : we override the target-dependent .ldi file
//----------------------------------------------------------------------------

MEMORY
{
#ifdef CYGPKG_HAL_ARM_ARM9_P5DEV
  itcm : ORIGIN = CYGMEM_REGION_itcm, LENGTH = CYGMEM_REGION_itcm_SIZE
  dtcm : ORIGIN = CYGMEM_REGION_dtcm, LENGTH = CYGMEM_REGION_dtcm_SIZE
  intram : ORIGIN = CYGMEM_REGION_intram, LENGTH = CYGMEM_REGION_intram_SIZE
  ebc : ORIGIN = CYGMEM_REGION_intram_SHADOW, LENGTH = BASEBAND_RESERVED
#endif
  ram : ORIGIN = CYGMEM_REGION_ram, LENGTH = CYGMEM_REGION_ram_SIZE
  // virtual memory space
  virtual : ORIGIN = CYGMEM_REGION_virtual, LENGTH = CYGMEM_REGION_virtual_SIZE
#if defined(CYGPKG_BASEBAND) && defined(CYGPKG_HAL_ARM_ARM9_VPB926EJS)
  intram : ORIGIN = 0xc0000000, LENGTH = 96K
#endif
}

SECTIONS
{
  SECTIONS_BEGIN
#ifndef CYGPKG_HAL_ARM_ARM9_P5DEV
    SECTION_fixed_vectors (ram, 0x20, LMA_EQ_VMA)
    // leave some room for RedBoot stuff, skip 256 KB
    SECTION_rom_vectors (ram, 0x40000, LMA_EQ_VMA)
#else
    // TCM sections
    SECTION_fixed_vectors (itcm, 0x20, LMA_EQ_VMA)
    // skip 1st kB (no access in user mode)
    SECTION_itcm(itcm, 0x400, LMA_EQ_VMA)
    SECTION_dtcm(dtcm, CYGMEM_REGION_dtcm, LMA_EQ_VMA)

    // Fast code & data go into INTRAM, skip Baseband DMA buffers
    SECTION_text_fast(intram,CYGMEM_REGION_intram+BASEBAND_RESERVED,LMA_EQ_VMA)
    SECTION_data_fast(intram, ALIGN(0x4), LMA_EQ_VMA)

    // Baseband DMA buffers
    SECTION_bss_BaseBand(ebc, CYGMEM_REGION_intram_SHADOW, LMA_EQ_VMA)

    // Skip room for main MMU table + 1 page table + RedBoot stuff
    SECTION_rom_vectors (ram, CYGMEM_REGION_ram + LOAD_OFFSET_RAM, LMA_EQ_VMA)
    SECTION_RELOCS (ram, ALIGN (0x1), LMA_EQ_VMA)
#endif

    // eCos code and read_only sections
    MMU_ALIGN(text_ecos,
          SECTION_text (ram, ALIGN (0x4), LMA_EQ_VMA)
          SECTION_fini (ram, ALIGN (0x4), LMA_EQ_VMA)
          SECTION_rodata (ram, ALIGN (0x4), LMA_EQ_VMA)
          SECTION_rodata1 (ram, ALIGN (0x4), LMA_EQ_VMA)
          SECTION_fixup (ram, ALIGN (0x4), LMA_EQ_VMA)
          SECTION_gcc_except_table (ram, ALIGN (0x4), LMA_EQ_VMA))

    // Where to put eCos data and bss ? If startup != RAM, eCos will duplicate
    // its data section into RAM, therefore we should have LMA != VMA...
    // On the other hand, LMA_EQ_VMA actually works even if startup != RAM,
    // because the duplication is harmless in that case (cf. vectors.S).
    MMU_ALIGN(data_ecos,
          SECTION_data(ram, ALIGN (0x4), LMA_EQ_VMA)
              SECTION_heap(ram, ALIGN (0x4), LMA_EQ_VMA)
          SECTION_bss (ram, ALIGN (0x4), LMA_EQ_VMA))

    // virtual processes mappings
    VP(virtual, CYGMEM_REGION_virtual, FOLLOW_MMU_ALIGNED(.bss), 01)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data01), 02)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data02), 03)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data03), 04)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data04), 05)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data05), 06)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data06), 07)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data07), 08)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data08), 09)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data09), 10)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data10), 11)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data11), 12)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data12), 13)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data13), 14)
    VP(virtual, ALIGN (MMU_SECTION_SIZE), FOLLOW_MMU_ALIGNED(.data14), 15)

    CYG_LABEL_DEFN(__memprot_rom_end) = FOLLOW_MMU_ALIGNED(.data15);
    CYG_LABEL_DEFN(__free_ram_start) = FOLLOW_MMU_ALIGNED(.data15);

    // additional eCos heap in virtual memory
    MMU_ALIGN(heap_ecos,SECTION_vp_heap(virtual,ALIGN(0x4),LMA_EQ_VMA,ecos))

#if defined(CYGPKG_BASEBAND) && defined(CYGPKG_HAL_ARM_ARM9_VPB926EJS)
    // Baseband RAM
    SECTION_data_BaseBand( intram, 0xc0000000, LMA_EQ_VMA )
#endif

    SECTIONS_END
}
