/**
 * ============================================================================
 * @file  diag.h
 * @brief A few macros used for printing debug/info/error messages.
 *
 * Copyright (C) 2004,2005 Parrot S.A.
 *
 * @author     ivan.djelic@parrot.fr
 * @date       2004-05-04
 * ============================================================================
 */

#ifndef CYGONCE_MEMPROT_DIAG_H
#define CYGONCE_MEMPROT_DIAG_H

#include <cyg/infra/diag.h>
#include <pkgconf/memprot.h>

// ANSI color escape sequences
#define DIAG_BLACK   "\033[30m"
#define DIAG_RED     "\033[31m"
#define DIAG_GREEN   "\033[32m"
#define DIAG_YELLOW  "\033[33m"
#define DIAG_BLUE    "\033[34m"
#define DIAG_MAGENTA "\033[35m"
#define DIAG_CYAN    "\033[36m"
#define DIAG_WHITE   "\033[37m"
#define DIAG_NONE    "\033[00m"

#define color_diag_printf( _color_, fmt, args... ) { \
 diag_write_string( _color_ "memprot: " );           \
 diag_printf( fmt , ##args );                        \
 diag_write_string( DIAG_NONE );                     \
}

#ifdef CYGDBG_MEMPROT_COLOR_DIAG
#  define DEBUG( fmt, args... )  color_diag_printf( DIAG_GREEN, (fmt), ##args )
#  define INFO( fmt, args... )   color_diag_printf( DIAG_RED, (fmt), ##args )
#else
#  define DEBUG( fmt, args... )  diag_printf( (fmt) , ##args )
#  define INFO( fmt, args... )   diag_printf( (fmt) , ##args )
#endif // COLOR_DIAG

#ifndef CYGDBG_MEMPROT_DEBUG
#  undef DEBUG
#  define DEBUG( fmt, args... )  { }
#endif

#ifndef CYGDBG_MEMPROT_DIAG
#  undef INFO
#  define INFO( fmt, args... )   { }
#endif

#endif // CYGONCE_MEMPROT_DIAG_H
