//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * ============================================================================
 * @file  memprot_vpb926ejs.c
 * @brief Versatile board memory protection specific setup functions
 *
 * Copyright (C) 2004,2005 Parrot S.A.
 *
 * @author     ivan.djelic@parrot.fr
 * @date       2004-05-04
 * ============================================================================
 */

#include "mmu.h"
#include "diag.h"

#ifdef CYGPKG_HAL_ARM_ARM9_VPB926EJS

// get memory layout
#include <pkgconf/system.h>
#include CYGHWR_MEMORY_LAYOUT_H

#include "memprot.h"

/** VPB926EJS specific initialisation.
 */
void memprot_vpb926ejs_setup(CYG_ADDRESS ram_base, CYG_ADDRESS ram_top) {
  // map first 4 KB page (VSRs) not accessible in user context
  memprot_map( 0x00000000, CYGMEM_REGION_ram_PHYS, 0x1000, AP_RW_NA, 1, 0 );
  // don't map RedBoot code + data
  //memprot_map( 0x00008000, V2P(0x00008000), 0x40000-0x8000, AP_NA_NA, 0, 0 );
  // flat map rest of RAM (uncacheable and unbufferable)
  // this is required because MMU page tables are dynamically allocated in RAM
  memprot_map( ram_base, ram_base, ram_top-ram_base, AP_RW_NA, 0, 0 );
  // remap SDRAM bank with cache and buffer disabled
  memprot_map( 0x04000000, 0x0, CYGMEM_REGION_ram_SIZE, AP_RW_NA, 0, 0 );
  // flat map FLASH (cacheable and bufferable) read-only
  memprot_map( 0x34000000, 0x34000000, 0x04000000, AP_RW_RO, 1, 0 );
  // flat map SRAM (uncacheable and unbufferable)
  memprot_map( 0x38000000, 0x38000000, 0x00200000, AP_RW_NA, 0, 0 );
  // Versatile board 35 MHz counter
  memprot_map( 0x10000000, 0x10000000, 0x1000, AP_RW_RO, 0, 0 );
  // Versatile board I/O registers
  memprot_map( 0x10001000, 0x10001000, 0x1fff000, AP_RW_NA, 0, 0 );
  // AMBA peripherals
  memprot_map( 0xc0000000, 0xc0000000, 0x30000, AP_RW_NA, 0, 0 );
  // EBC registers on FPGA baseband
  memprot_map( 0xc1000000, 0xc1000000, 0x10000, AP_RW_NA, 0, 0 );
  memprot_map( 0xd0000000, 0xd0000000, 0xb0000, AP_RW_NA, 0, 0 );
  // FIXME: this one is temporarily mapped RW for direct GPIO access
  memprot_map( 0xe0000000, 0xe0000000, 0x70000, AP_RW_RW, 0, 0 );
}

#endif
