//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * ============================================================================
 * @file  syscall.h
 * @brief A few macros/functions used to check system calls arguments.
 *
 * Memory protection system call handling.
 *
 * Copyright (C) 2004,2005 Parrot S.A.
 *
 * @author     ivan.djelic@parrot.fr
 * @date       2004-05-05
 * ============================================================================
 */
#ifndef CYGONCE_MEMPROT_SYSCALL_H
#define CYGONCE_MEMPROT_SYSCALL_H

#include "mmu.h"
#include "thread.h"

#define USER_READ     0
#define USER_WRITE    1
#define SYSTEM_READ   2
#define SYSTEM_WRITE  3

int __check_access( CYG_ADDRESS addr, cyg_uint32 size, int access );
int __check_owner_domain( CYG_ADDRESS addr, cyg_uint32 size );

//! A macro used to declare a pseudo-function which is in reality a SWI call
#define SWI_DECLARE(_syscall_,_gate_) \
void __swi_##_syscall_ (void) __attribute__ ((naked)); \
void __swi_##_syscall_ (void) { asm( "swi\n\t.word " #_gate_ "\n"); }

#define __check_system_read( _a_, _s_ ) \
 __check_access(((CYG_ADDRESS)_a_), ((cyg_uint32)_s_), SYSTEM_READ )

#define __check_system_write( _a_, _s_ ) \
 __check_access(((CYG_ADDRESS)_a_), ((cyg_uint32)_s_), SYSTEM_WRITE )

#define __check_user_read( _a_, _s_ ) \
 __check_access(((CYG_ADDRESS)_a_), ((cyg_uint32)_s_), USER_READ )

#define __check_user_write( _a_, _s_ ) \
 __check_access(((CYG_ADDRESS)_a_), ((cyg_uint32)_s_), USER_WRITE )

#define __check_owner( _a_, _s_ ) \
 __check_owner_domain(((CYG_ADDRESS)_a_), ((cyg_uint32)_s_))

#endif // CYGONCE_MEMPROT_SYSCALL_H
