//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
******************************************************************************
* @file vp_layout.h
* @brief Virtual Process memory layout structure
*
* Memory Protection
*
* Copyright (C) 2004,2005 Parrot S.A.
*
* @author     ivan.djelic@parrot.fr
* @date       2004-05-26
******************************************************************************
*/
#ifndef CYGONCE_MEMPROT_VP_LAYOUT_H
#define CYGONCE_MEMPROT_VP_LAYOUT_H

#include <cyg/infra/cyg_type.h>

//! This structure is used to map a process in virtual memory
typedef struct _cygmem_vp_layout_t {

  //! Virtual memory (vm) start : should be MMU_SECTION (1 MB) aligned
  CYG_ADDRESS vm_start;
  //! Virtual memory (vm) end : should be MMU_SECTION (1 MB) aligned
  CYG_ADDRESS vm_end;

  //! Read-only area (text + rodata) virtual start : should be PAGE aligned
  CYG_ADDRESS ro_start;
  //! Read-only area (text + rodata) virtual end : should be PAGE aligned
  CYG_ADDRESS ro_end;

  //! Read/write area (data + bss) virtual start : should be PAGE aligned
  CYG_ADDRESS rw_start;
  //! Read/write area (data + bss) virtual end : should be PAGE aligned
  CYG_ADDRESS rw_end;

  //! Heap area virtual start
  CYG_ADDRESS heap_start;
  //! Heap area virtual top : this should be equal to heap_start at start-up.
  CYG_ADDRESS heap_top;
  //! Heap area virtual end : should be PAGE (4 KB) aligned
  CYG_ADDRESS heap_end;

  //! Stacks area virtual start
  CYG_ADDRESS stacks_start;
  //! Stacks area virtual end : should be PAGE (4 KB) aligned
  CYG_ADDRESS stacks_end;

  //! Data section virtual start (inside the read/write area)
  CYG_ADDRESS data_start;
  //! Data section virtual end (inside the read/write area)
  CYG_ADDRESS data_end;

  //! BSS section virtual start (inside the read/write area)
  CYG_ADDRESS bss_start;
  //! BSS section virtual start (inside the read/write area)
  CYG_ADDRESS bss_end;

  //! Read-only area physical start : this will be mapped in ROM/FLASH.
  CYG_ADDRESS rom_ro_start;
  //! Read-only area physical end : this will be mapped in ROM/FLASH.
  CYG_ADDRESS rom_ro_end;

  //! Data section physical start : this is copied from ROM/FLASH to RAM.
  CYG_ADDRESS rom_data_start;
  //! Data section physical end
  CYG_ADDRESS rom_data_end;
} cygmem_vp_layout_t;


#endif // CYGONCE_MEMPROT_VP_LAYOUT_H
