/*
 * RAM allocator TEST
 *
 */

#include "ram_alloc.h"

#define SIZE_1MB (1 << 20)

void ram_init( CYG_ADDRESS base, CYG_ADDRESS top );
CYG_ADDRESS ram_allocate(size_t size);
void ram_free( CYG_ADDRESS addr, size_t size );

int main(void) {
  int i;
  CYG_ADDRESS addr, ram_base, ram_base_mb;

  // allocate 3 MB for test

#define RAM_SIZE (3*SIZE_1MB)

  ram_base = (CYG_ADDRESS)malloc(RAM_SIZE);
  ram_base_mb = (ram_base+SIZE_1MB+0x1000) & ~(SIZE_1MB-1);
  ram_init( ram_base_mb-0x1000, ram_base_mb + (RAM_SIZE-SIZE_1MB));

  // allocate 64 KB
  addr = ram_allocate(64 << 10);
  CYG_ASSERT( addr, "Allocation failed !" );

  // and free only 4 KB at a time, in a pseudo-random order
  for ( i = 0; i < 16; i++ )
    ram_free( addr+(i^0xa)*4096, 4096 );

  // free another 4 KB in a section
  addr = ram_allocate(1024 << 10);
  CYG_ASSERT( addr, "Allocation failed !" );
  ram_free( addr, 4096 );

  return 0;
}
