#include <pkgconf/system.h>
#include <pkgconf/net.h>
#include <task_priorities.h>

#define NBNEP 1
#define _KERNEL

#include <sys/param.h>
#define suser(x) 0
#include <sys/mbuf.h>
#include <sys/socket.h>
#include <sys/sockio.h>
#include <sys/time.h>
#include <sys/malloc.h>
#include <sys/mbuf.h>

#include <net/if.h>
#include <net/if_types.h>
#include <net/netisr.h>
#include <net/if_arp.h>
#include <net/if_dl.h>
#include <net/route.h>
#include <net/if_bnep.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/in_var.h>
#include <netinet/ip.h>
#include <netinet/if_ether.h>

#define BNEP_MTU 1500
#define BNEP_HDRLEN 3

#define SC_DEBUG    0x00010000  /* enable debug messages */
#define SC_LOG_INPKT    0x00020000  /* log contents of good pkts recvd */
#define SC_LOG_OUTPKT   0x00040000  /* log contents of pkts sent */
#define SC_LOG_RAWIN    0x00080000  /* log all chars received */
#define SC_LOG_FLUSH    0x00100000  /* log all chars flushed */
#define SC_MASK     0x0fff00ff  /* bits that user can change */

 /* State bits in sc_flags, not changeable by user.
 */
#define SC_TIMEOUT  0x00000400  /* timeout is currently pending */
#define SC_TBUSY    0x10000000  /* xmitter doesn't need a packet yet */
#define SC_PKTLOST  0x20000000  /* have lost or dropped a packet */
#define SC_FLUSH    0x40000000  /* flush input until next PPP_FLAG */

#define M_ERRMARK   0x4000  /* steal a bit in mbuf m_flags */

#define DEBUG_IFBNEP_DISABLED

// the following will activate the resolution of the bluetooth adress from the IP using arp table
// if not enabled the packet will be sent on all BNEP connections
// this flag will be really relevant when GN or NAP is enabled
#define PREPEND_GATEWAY

/** Function in charge of sending packets
 */
//static void (*SendPacket)(cyg_uint16 *,cyg_uint16,void *, cyg_uint16, cyg_uint16);
static cyg_bnep_sender_func_t SendPacket;

#define BLP_BNEP_PACKET_START   0
#define BLP_BNEP_PACKET_BODY    1
#define BLP_BNEP_PACKET_END     2

int bnepoutput( struct ifnet *ifp, struct mbuf *m0, struct sockaddr *dst, struct rtentry *rtp);
int bnep_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data);

struct bnep_softc {
    struct  ifnet sc_if;    /* network-visible interface */
    u_int   sc_flags;       /* control/status bits; see if_bnep.h */
    struct arpcom sc_arpcom;
};

struct bnep_softc bnep_softc[NBNEP];

void bnepattach(void *);
PSEUDO_SET(bnepattach, if_bnep);

#define show_packet(abel,proto,data,len)
#define SIN(s) ((struct sockaddr_in *)s)
#define SDL(s) ((struct sockaddr_dl *)s)
#define SIN6(s) ((struct sockaddr_in6 *)s)

#ifdef DEBUG_IFBNEP
#define BNEP_TRACE(fmt...) printf(fmt)
#else
#define BNEP_TRACE(fmt...)
#endif

#define BNEP_SENDER_STACK_SIZE (CYGNUM_HAL_STACK_SIZE_TYPICAL)

static void bnep_sender_thread(cyg_addrword_t data);

static unsigned char BNEP_Sender_Stack[BNEP_SENDER_STACK_SIZE];
cyg_handle_t bnep_thread_handle;
cyg_thread bnep_thread;
cyg_flag_t bnep_data_ready;

cyg_mutex_t bnep_mutex;

static void dummy_sender(cyg_uint16 *a, cyg_uint16 b, void *c, cyg_uint16 d, cyg_uint16 e)
{
}

/*
 * Called from boot code to establish bnep interfaces.
 */
void bnepattach(void *dummy)
{
    struct bnep_softc *sc;
    int i = 0;

    SendPacket=dummy_sender;
    for (sc = bnep_softc; i < NBNEP; sc++) {
        sc->sc_if.if_name = "bnep";
        sc->sc_if.if_unit = i++;
        sc->sc_if.if_mtu = BNEP_MTU;
        sc->sc_if.if_flags = IFF_BROADCAST | IFF_DEBUG | IFF_MULTICAST;
        sc->sc_flags= SC_LOG_OUTPKT | SC_LOG_INPKT;
        sc->sc_if.if_type = IFT_ETHER;
        sc->sc_if.if_hdrlen = BNEP_HDRLEN;
        sc->sc_if.if_output = bnepoutput;
        sc->sc_if.if_ioctl = bnep_ioctl;

        sc->sc_if.if_snd.ifq_maxlen = IFQ_MAXLEN;
        sc->sc_if.if_softc=(void*)0x800;            /* use this field to store the protocol */
        if_attach(&sc->sc_if);
    }

    cyg_mutex_init(&bnep_mutex);

    cyg_flag_init(&bnep_data_ready);

    /* create a thread in charge of fetching mbuf from queue to assemble then and send them to BLUES
     */
    cyg_thread_create(PRIO_ECOS_NET_BNEP_SENDER, bnep_sender_thread,
                      0, "Net_BNEP_sender",
                      BNEP_Sender_Stack, BNEP_SENDER_STACK_SIZE,
                      &bnep_thread_handle, &bnep_thread);

    cyg_thread_resume(bnep_thread_handle);
}

void cyg_bnep_set_sender(cyg_bnep_sender_func_t fun)
{
    SendPacket=fun;
}

//
// This function supports "I/O control" operations on an interface.
//
int  bnep_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
{
    struct bnep_softc *sc = ifp->if_softc;
    struct ifaddr *ifa = (struct ifaddr *) data;
    struct ifreq *ifr = (struct ifreq *)data;
    int     s, error = 0;

    s = splnet();

    switch (cmd) {

    case SIOCSIFADDR:
        ifp->if_flags |= IFF_UP;

        switch (ifa->ifa_addr->sa_family) {
        case AF_INET:
            sc->sc_arpcom.ac_if=*ifp;
            sc->sc_arpcom.ac_if.if_softc=(void*)0x806;          /* use this field to store the protocol */
            ifp->if_flags |= IFF_RUNNING;
            memset(&sc->sc_arpcom.ac_enaddr, 0xAA, 6);      /* TODO we must get the bluetooth address */
            arp_ifinit(&sc->sc_arpcom, ifa);
            break;
        case AF_INET6:
            sc->sc_arpcom.ac_if=*ifp;
            sc->sc_arpcom.ac_if.if_softc=(void*)0x86DD;         /* use this field to store the protocol */
            ifp->if_flags |= IFF_RUNNING;
            memset(&sc->sc_arpcom.ac_enaddr, 0xAA, 6);          /* TODO we must get the bluetooth address */
            arp_ifinit(&sc->sc_arpcom, ifa);
            break;
        }
        break;

    case SIOCGIFHWADDR:
        // Get hardware (MAC) address
        ifr->ifr_hwaddr.sa_family = AF_INET;
        bcopy(&sc->sc_arpcom.ac_enaddr, &ifr->ifr_hwaddr.sa_data, 6 /*ETHER_ADDR_LEN*/ );
        break;

    case SIOCSIFHWADDR:
        // Set hardware (MAC) address
        bcopy(&ifr->ifr_hwaddr.sa_data, &sc->sc_arpcom.ac_enaddr, 6/*ETHER_ADDR_LEN*/);
        break;

    default:
        error = EINVAL;
        break;
    }

    splx(s);
    return (error);
}

#ifdef DEBUG_IFBNEP
static void show_address(struct sockaddr *addr)
{
    int i;
    char c;

    BNEP_TRACE("sockaddr: sa_len=%d: ", addr->sa_len);

    switch (addr->sa_family) {
    case AF_INET:
        BNEP_TRACE("AF_INET sa_port=%d sa_addr=%s", SIN(addr)->sin_port, inet_ntoa(SIN(addr)->sin_addr));
        break;
    case AF_INET6:
        BNEP_TRACE("AF_INET6 sa_port=%d sin6_addr", SIN6(addr)->sin6_port);
        c='=';
        for (i=0; i<16; i++) {
            BNEP_TRACE("%c%02X", c, ((char*)(&(SIN6(addr)->sin6_addr)))[i]);
            c=':';
        }
        break;
    default:
        BNEP_TRACE("family=%d ", addr->sa_family);
        for (i=0; i< 16; i++) {
            BNEP_TRACE(" %02X", addr->sa_data[i]);
        }
        break;
    }
    BNEP_TRACE("\n");
}
#else
#define show_address(x)
#endif

/*
 * Send a packet
 * in fact queue it for delayed sending the a sender thread
 */
int bnepoutput(struct ifnet *ifp, struct mbuf *m0, struct sockaddr *dst, struct rtentry *rtp)
{
    register struct bnep_softc *sc = &bnep_softc[ifp->if_unit];
    int error;
    struct ifqueue *ifq;
    struct ip *ip;
    struct mbuf *maddr;

    /* check if interface is active */
    if ((ifp->if_flags & IFF_RUNNING) == 0 ||
        ((ifp->if_flags & IFF_UP) == 0 && dst->sa_family != AF_UNSPEC)) {
        error = ENETDOWN;   /* sort of */
        goto bad;
    }

#ifdef DEBUG_IFBNEP_DISABLED
    if (sc->sc_flags & SC_LOG_OUTPKT) {
        int len = 0;
        struct mbuf *m;

        for (m = m0; m != 0; m = m->m_next)
            len += m->m_len;

        BNEP_TRACE("bnep%d output len=%d\n", ifp->if_unit, len);
    }

    BNEP_TRACE("Protocol=%04X\n", ifp->if_softc);
#endif

    ifq = &ifp->if_snd;

    /* if the queue is full drop the packet */
    if (IF_QFULL(ifq) && dst->sa_family != AF_UNSPEC) {
        BNEP_TRACE("DROP packet\n");
        IF_DROP(ifq);
        error = ENOBUFS;
        goto bad;
    }
    cyg_mutex_lock(&bnep_mutex);

    ip = mtod(m0, struct ip *);

    //show_packet("OUTGOING", protocol, ip, m0->m_len);

#ifdef PREPEND_GATEWAY
    /* store first hop address in front of header
     */
    MGET(maddr, M_WAIT, MT_DATA);
    maddr->m_len=sizeof(struct sockaddr);
    *mtod(maddr,struct sockaddr*)=*dst;
    maddr->m_next=m0;
    m0=maddr;
#endif

    /*
     * Put the packet on the queue
     */
    IF_ENQUEUE(ifq, m0);

    // notify the sending thread that there is a packet to send
    cyg_flag_setbits(&bnep_data_ready, 1);

    cyg_mutex_unlock(&bnep_mutex);

    // keep track of the last change
    getmicrotime(&ifp->if_lastchange);
    return (0);

bad:
    m_freem(m0);
    return (error);
}

/*
 * Get a packet to send.  This procedure is intended to be called at
 * splsoftnet
 */
inline struct mbuf * bnep_dequeue(struct bnep_softc *sc)
{
    struct mbuf *m;

    /*
     * Grab a packet to send
     */
    IF_DEQUEUE(&sc->sc_if.if_snd, m);
    return m;
}

#ifdef DEBUG_IFBNEP
/* transform an internet address into a almost human readable form
 */
static inline char *stringize_addr(cyg_uint32 addr, char *p)
{
    struct in_addr ia;
    ia.s_addr=addr;
    return strcpy(p, inet_ntoa(ia));
}

/* display a IP header
 */
static void show_ip_header(struct ip *ip)
{
    char tmp1[16], tmp2[16];
    char *format="IP HEADER v%d, hlen=%d, TOS=%d, LEN=%d, ID=%d, OFF=%d, TTL=%d, PROTO %X, SRC=%s DST=%s\n";

    if (ip->ip_v==4)
        printf(format, ip->ip_v, ip->ip_hl*4, ip->ip_tos, ntohs(ip->ip_len), ntohs(ip->ip_id), ntohs(ip->ip_off)&0xfff, ip->ip_ttl, ip->ip_p, stringize_addr(ip->ip_src.s_addr,tmp1), stringize_addr(ip->ip_dst.s_addr, tmp2));
    else {
        printf("IPv6 packet?\n");
    }
}

#undef show_packet
static void show_packet(char *label, cyg_uint16 proto, char *data, int len)
{
    int j=0, k;

    printf("%s PACKET\n", label);
    if (proto==0x800) {
        printf("IP Header\n");
        for (; j<sizeof(struct ip); j++) {
            char c=data[j];

            printf("%02X ", c);
        }
        printf("\n");
        show_ip_header((struct ip *)data);
    }
    else if (proto==0x806)
    {
        printf("ARP Header\n");
    }
    printf("PAYLOAD\n");
    for (k=0; j<len; j++, k++) {
        char c=data[j];

        if (1 || c<0x20 || c>0x7E)
            printf("%02X ", c);
        else
            printf("%02X(%c) ", c, c);
        if (k>=16) {
            printf("...\n");
            return;
        }
    }
    printf("\n");
}
#endif

/* dirty hack....
 * I borowed some code from arp in if_ether.c to resolve an IP address to a BT address
 * the stack should not change but beware...
 */
struct llinfo_arp {
    LIST_ENTRY(llinfo_arp) la_le;
    struct  rtentry *la_rt;
};
struct llinfo_arp * arplookup( u_long addr, int create, int proxy);

/*
 * Resolve an IP address into a bluetooth address
 */
int bt_resolve( struct sockaddr *dst, u_char *desten)
{
    struct rtentry *rt;
    struct llinfo_arp *la = 0;
    struct sockaddr_dl *sdl;

    la = arplookup(SIN(dst)->sin_addr.s_addr, 1, 0);
    if (la)
        rt = la->la_rt;
    if (la == 0 || rt == 0)
        return 0;

    sdl = SDL(rt->rt_gateway);
    /*
     * Check the address family and length is valid, the address
     * is resolved; otherwise, try to resolve.
     */
    if (sdl->sdl_family == AF_LINK && sdl->sdl_alen != 0) {
        bcopy(LLADDR(sdl), desten, sdl->sdl_alen);
        return 1;
    }
    else
        return 0;
}

/*
 * Software interrupt routine, called at spl[soft]net.
 */
static void bnep_sender_thread(cyg_addrword_t data)
{
    struct bnep_softc *sc;
    struct mbuf *m;
    struct mbuf *m0;
    int i;

    while (1) {
        // wait for some data to be queued
        cyg_flag_wait(&bnep_data_ready, 1, CYG_FLAG_WAITMODE_OR);

        sc = bnep_softc;
        for (i = 0; i < NBNEP; ++i, ++sc) {
            for (;;) {
                struct ip *ip;
                cyg_uint16 protocol;
                struct sockaddr dst;
                cyg_uint8 bt_addr[6];
                cyg_uint16 *dest_addr=(cyg_uint16 *)bt_addr;

                cyg_mutex_lock(&bnep_mutex);
                m=bnep_dequeue(sc);
                cyg_mutex_unlock(&bnep_mutex);

                if (m == NULL) break; /* no packet to send */

#ifdef PREPEND_GATEWAY
                //BNEP_TRACE("sender:\n");
                dst=*mtod(m, struct sockaddr *);
                show_address(&dst);

                //BNEP_TRACE("bef %08X %d -- %08X -- %08X\n", m, m->m_len, m->m_next, m->m_next->m_next);
                m0=m;
                m=m_split(m, sizeof(struct sockaddr), M_WAIT);
                m_free(m0);
                //BNEP_TRACE("aft %08X %d\n", m, m->m_len);

                memset(bt_addr,0, sizeof(bt_addr));

                /* resolve the next ip address to a bluetooth address using arp
                 */
                if (!bt_resolve(&dst, bt_addr))
#endif
                    dest_addr=NULL;

                BNEP_TRACE("BT Addr: %02X:%02X:%02X:%02X:%02X:%02X\n",
                            bt_addr[0], bt_addr[1],
                            bt_addr[2], bt_addr[3],
                            bt_addr[4], bt_addr[5]);

                {
                    cyg_uint16 j=sizeof(bt_addr)/sizeof(bt_addr[0]);
                    cyg_uint16 i=0;

                    /* we must reverse the order of the address
                    */
                    while (i<j) {
                        cyg_uint8 t=bt_addr[i];
                        bt_addr[i++]=bt_addr[--j];
                        bt_addr[j]=t;
                    }
                }

                /* we resolved the address of destination
                 * send the packet
                 * */
                ip = mtod(m, struct ip *);

                show_packet("OUTGOING", protocol, (void*)ip, m->m_len);

                // BUGBUG it is not a very clean way....
                if (ip->ip_v==4)
                    protocol=0x800;
                else if (ip->ip_v==6)
                    protocol=0x86DD;
                else
                    protocol=0x806;

                /* send each mbuf
                 * reassembly is done by the BNEP module
                 */
                SendPacket(dest_addr, protocol, NULL, 0, BLP_BNEP_PACKET_START);
                for (m0 = m; m0 != 0; m0 = m0->m_next) {
                    SendPacket(dest_addr, 0, mtod(m0, void*), m0->m_len,
                                        BLP_BNEP_PACKET_BODY);
                }
                SendPacket(dest_addr, 0,  NULL, 0, BLP_BNEP_PACKET_END);

                /* free the buf list */
                m_freem(m);
            }
        }
        cyg_flag_maskbits(&bnep_data_ready, 0);
    }
}

/*
 * BNEP packet input routine.
 * this function should be called when data is available fror the stack
 */
void bneppktin(cyg_uint16 protocol, struct mbuf *m)
{
    // not declared in a header so do it here
    extern struct   ifqueue arpintrq;

    int s = splimp();
    switch (protocol) {
    case 0x86DD:    /* IPv6 packet */
    case 0x800:     /* IPv4 packet */
        IF_ENQUEUE(&ipintrq, m);
        schednetisr(NETISR_IP);
        break;
    case 0x806:     /* ARP packet */
        IF_ENQUEUE(&arpintrq, m);
        schednetisr(NETISR_ARP);
        break;
    }
    splx(s);
}

void cyg_bnep_data_available(cyg_uint16 ifn, cyg_uint16 protocol, void *data, int len)
{
    show_packet("INCOMING", protocol, data,len);
    bneppktin(protocol, m_devget(data, len, 0, &bnep_softc[ifn].sc_if , NULL));
}
