/**
*******************************************************************************
* @file  process.h
* @brief Dynamo process public interface
*
* Copyright (C) 2007 Parrot S.A.
*
* @author     ivan.djelic@parrot.fr
* @date       2007-02-04
*******************************************************************************
*/
#ifndef CGYONCE_DYNAMO_PROCESS_H
#define CGYONCE_DYNAMO_PROCESS_H

#include <cyg/kernel/kapi.h>
#include <errno.h>

//! A private virtual memory segment structure
struct segment {
    CYG_ADDRESS base;
    cyg_uint32 size;
};

//! A private process structure
typedef struct cyg_process {
    int status;                                 //! Process status
    int domain;                                 //! User domain
    const char *name;
    const char *file;
    const char *entry;
    cyg_addrword_t sched_info;
    int nb_threads;
    int max_threads;
    cyg_uint32 used_mem;
    cyg_uint32 max_mem;
    struct segment ro;
    struct segment rw;
    struct segment heap;
    int heap_lock_count;
    cyg_mutex_t mutex;
} cyg_process;

void dynamo_init(void);

Cyg_ErrNo cyg_process_create( cyg_addrword_t sched_info,
                              const char *name,
                              const char *file,
                              const char *entry,
                              cyg_uint32 domain,
                              cyg_uint32 max_threads,
                              cyg_uint32 max_mem,
                              cyg_handle_t *handle,
                              cyg_process *this );

void cyg_process_resume(cyg_handle_t process);

#endif
