//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * @file   rtmon_hal.c
 * @brief  RTmon HAL-dependent routines
 *
 * @author ivan.djelic@parrot.com
 * @date   2008-04-04
 *
 * This file is licensed under the eCos Public License. A copy of this license
 * can be obtained from:
 * http://ecos.sourceware.org/license-overview.html
 *
 * The Original Code is eCos - Embedded Configurable Operating System,
 *
 * Portions created by Parrot S.A. are
 * Copyright (C) 2004-2008 Parrot S.A.
 */

#include <pkgconf/system.h>
#if defined(CYGPKG_HAL_ARM_ARM9_P6DEV)

#include <cyg/kernel/kapi.h>
#include <cyg/infra/diag.h>         // diagnostic output
#include <pkgconf/rtmon.h>
#include <cyg/rtmon/rtmon.h>
#include <cyg/rtmon/rtmon_common.h>
#include <cyg/rtmon/rtmon_internal.h>
#include <cyg/rtmon/rtmon_hal.h>
#include <cyg/crc/crc.h>
#include <cyg/hal/hal_diag.h>

cyg_uint32 p6i_rtmon_timer_period;
cyg_uint32 p6i_rtmon_timefact_mult;
cyg_uint32 p6i_rtmon_timefact_div;

//-----------------------------------------------------------------------------
// Parrot6/6i platform

void rtmon_hal_start_timer(void)
{
    /* do nothing, we are using the system timer */
}

void rtmon_hal_init_var(void)
{
	cyg_uint32 speed;

    /* Get AHB clock */
    HAL_READ_UINT32(PARROT6_SYS + _P6_SYS_SPEED, speed);

    switch (speed & 0x1F)
    {
        case 8: //156 MHz
            p6i_rtmon_timer_period = 48750;
            p6i_rtmon_timefact_mult = 80;
            p6i_rtmon_timefact_div  = 39;
            break;

        case 14: //130 MHz
            p6i_rtmon_timer_period = 40625;
            p6i_rtmon_timefact_mult = 32;
            p6i_rtmon_timefact_div  = 13;
            break;

        case 16: //138.6 MHz
            p6i_rtmon_timer_period = 43333;
            p6i_rtmon_timefact_mult = 100000;
            p6i_rtmon_timefact_div  =  43333;
            break;

        default:
            break;
    }
}

void rtmon_hal_stop_timer(void)
{
    /* do nothing, we are using the system timer */
}

//! declaration of the swi_handler routine from swi_handler.S
cyg_VSR_t swi_rtmon_handler;


void rtmon_hal_start_profiling(void)
{
    cyg_uint32 reg;
    cyg_uint32 ahb_clk, timer_50_micro_s;

    ahb_clk = pclock_get_rate(NULL) / 16;
    timer_50_micro_s = (cyg_uint32)(0.000050 * ahb_clk) - 1;

    /* allocate timer 2 */
    HAL_WRITE_UINT32(PARROT6_SYS+_P6_SYS_TIM2CTL, 0);
    HAL_WRITE_UINT32((PARROT6_VIC+_VIC_IntSelect),
                     1 << CYGNUM_HAL_INTERRUPT_TIMER2);
    cyg_interrupt_set_vsr(CYGNUM_HAL_VECTOR_FIQ, swi_rtmon_handler);
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_TIMER2);

    /* restart timer */
    HAL_WRITE_UINT32((PARROT6_SYS+_P6_SYS_TIM2LD ),
                     timer_50_micro_s );
    HAL_WRITE_UINT32((PARROT6_SYS+_P6_SYS_TIM2CTL),
                     P6_SYS_TIMXCTL_PERIODIC  |
                     P6_SYS_TIMXCTL_PRESCALE16|
                     P6_SYS_TIMXCTL_ENABLE);
    /* enable timer interrupt */
    HAL_READ_UINT32(PARROT6_SYS+_P6_SYS_ITEN, reg);
    reg |= P6_SYS_ITEN_TIM2;
    HAL_WRITE_UINT32(PARROT6_SYS+_P6_SYS_ITEN, reg);
}

void rtmon_hal_stop_profiling(void)
{
    // disable Timer 2
    cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_TIMER2);
    HAL_WRITE_UINT32(PARROT6_SYS+_P6_SYS_TIM2CTL, 0);
    // clear interrupt flag
    HAL_WRITE_UINT32(PARROT6_SYS+_P6_SYS_ITACK, P6_SYS_ITACK_TIM2);
}

#endif //CYGPKG_HAL_ARM_ARM9_P6DEV
