//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * @file   rtmon_hal.c
 * @brief  RTmon HAL-dependent routines
 *
 * @author ivan.djelic@parrot.com
 * @date   2008-04-04
 *
 * This file is licensed under the eCos Public License. A copy of this license
 * can be obtained from:
 * http://ecos.sourceware.org/license-overview.html
 *
 * The Original Code is eCos - Embedded Configurable Operating System,
 *
 * Portions created by Parrot S.A. are
 * Copyright (C) 2004-2008 Parrot S.A.
 */

#include <pkgconf/system.h>

#if defined(CYGPKG_HAL_ARM_ARM9_SMDK2412)

#include <cyg/kernel/kapi.h>
#include <cyg/infra/diag.h>         // diagnostic output
#include <pkgconf/rtmon.h>
#include <cyg/rtmon/rtmon.h>
#include <cyg/rtmon/rtmon_common.h>
#include <cyg/rtmon/rtmon_internal.h>
#include <cyg/rtmon/rtmon_hal.h>
#include <cyg/crc/crc.h>
#include <cyg/hal/hal_diag.h>

//-----------------------------------------------------------------------------
// Samsung s3c2412 platform

#define S3C2412_TCFG0_PSCALE1MSK        (0xFF<<8)
#define S3C2412_TCFG0_PSCALE1(_pscale_) (_pscale_<<8)

#define S3C2412_TCFG1_PCLKDIV2  0x0
#define S3C2412_TCFG1_MUX0_MASK (0x0000000F)
#define S3C2412_TCFG1_MUX4_MASK (0x000F0000)

#define S3C2412_TCON_T2START     (0x1<<12)
#define S3C2412_TCON_T2MANUALUPD (0x1<<13)
#define S3C2412_TCON_T2RELOAD    (0x1<<15)
#define S3C2412_TCON_T4START     (0x1<<20)
#define S3C2412_TCON_T4MANUALUPD (0x1<<21)
#define S3C2412_TCON_T4RELOAD    (0x1<<22)


/* timer 4 is for kernel */
#define S3C2412_TCON_OFFSET(timer) ((timer==0)?0:(4+timer*4))

void rtmon_hal_init_var(void)
{
    /* Nothing to init */
}

struct rtmon_hal_timer {
    const int          timer;            //!< timer number
    cyg_interrupt      irq_obj;          //!< interrupt object
    cyg_handle_t       irq_handle;       //!< interrupt handle
};

/* use s3c2412 timer 4 */
static struct rtmon_hal_timer rt = {.timer = 4};

static cyg_uint32 rtmon_hal_timer_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    /* Increment counter */
    RTMON_TICK();
    /* ack irq */
    cyg_drv_interrupt_acknowledge(vector);
    return CYG_ISR_HANDLED;
}

void rtmon_hal_start_timer(void)
{
    int irq;
    cyg_uint32 old, tcon, tcfg0, tcfg1, cnt;

    /* register timer interrupt */
    irq = CYGNUM_HAL_INTERRUPT_TIMER0 + rt.timer;
    cyg_drv_interrupt_create(irq,
                             4, /* priority */
                             (cyg_addrword_t)0,
                             rtmon_hal_timer_isr,
                             NULL, &rt.irq_handle, &rt.irq_obj);
    cyg_drv_interrupt_attach(rt.irq_handle);

    HAL_DISABLE_INTERRUPTS(old);

    /* configure and start timer */
    HAL_READ_UINT32(TCFG0, tcfg0);
    tcfg0 &= ~(S3C2412_TCFG0_PSCALE1MSK); // Prescaler1 is set to 0
    HAL_WRITE_UINT32(TCFG0, tcfg0);

    HAL_READ_UINT32(TCFG1, tcfg1);
    tcfg1 &= ~(S3C2412_TCFG1_MUX0_MASK << (rt.timer*4));
    tcfg1 |=  (0x3  << (rt.timer*4)); // PCLK / 2
    HAL_WRITE_UINT32(TCFG1, tcfg1);

    cnt = RTMON_HAL_TICKPERIOD /*((timer_base + freq/2 ) / freq) - 1*/;
    HAL_WRITE_UINT32(TCNTB4, cnt);

    HAL_READ_UINT32(TCON, tcon);
    if(rt.timer == 4){
        /* update timer */
        tcon &= ~(S3C2412_TCON_T4START | S3C2412_TCON_T4RELOAD | S3C2412_TCON_T4MANUALUPD);
        tcon |= (S3C2412_TCON_T4MANUALUPD | S3C2412_TCON_T4RELOAD);
        HAL_WRITE_UINT32(TCON, tcon);
        /* start timer */
        tcon &= ~(S3C2412_TCON_T4MANUALUPD);
        tcon |= (S3C2412_TCON_T4START);
        HAL_WRITE_UINT32(TCON, tcon);
    }

    HAL_RESTORE_INTERRUPTS(old);

    cyg_drv_interrupt_unmask(irq);
}

void rtmon_hal_stop_timer(void)
{
    cyg_uint32 tcon;

    if(rt.irq_handle != 0){
        /* stop timer */
        HAL_READ_UINT32(TCON, tcon);
        tcon &= ~(S3C2412_TCON_T4START);
        HAL_WRITE_UINT32(TCON, tcon);

        /* uninstall irq handler */
        cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_TIMER0 + rt.timer);
        cyg_drv_interrupt_detach(rt.irq_handle);
        cyg_drv_interrupt_delete(rt.irq_handle);
        rt.irq_handle = 0;
    }
}

void rtmon_hal_start_profiling(void)
{
    /* todo */
}

void rtmon_hal_stop_profiling(void)
{
    /* todo */
}

#endif /* CYGPKG_HAL_ARM_ARM9_SMDK2412 */
