
#include <stdio.h>

#include <cyg/hal/hal_intr.h> // Interrupt names
#include <cyg/hal/hal_io.h>
#include <cyg/kernel/kapi.h>  // thread
#include <cyg/kernel/diag.h>
#include <cyg/hal/hal_arch.h>           /* CYGNUM_HAL_STACK_SIZE_TYPICAL */

#include <unistd.h>
#include <cyg/fileio/fileio.h>
#include <dirent.h>
#include <errno.h>
#include <cyg/hal/hal_diag.h> // for HAL_DELAY_US
#include <cyg/hal/hal_io.h>     // for HAL_WRITE_UINT32

#include <assert.h>
#include <string.h>
#include <stdlib.h>

#include <sys/statvfs.h>

#include <cyg/rtmon/rtmon.h>
#include <cyg/rtmon/rtmon_internal.h>

#define NTHREADS 3
#define STACKSIZE ( CYGNUM_HAL_STACK_SIZE_TYPICAL + 4096 )

#define BLOCKING_WAIT 0x8FFFF // about 30 ms

/*  a mutex to protect calls the file_db */
cyg_mutex_t mtx_inv_prio;


/* STATICS */
static cyg_handle_t thread[NTHREADS];
static cyg_thread thread_obj[NTHREADS];
static char stack[NTHREADS][STACKSIZE];

void thread1(CYG_ADDRESS data);
void thread2(CYG_ADDRESS data);
void thread3(CYG_ADDRESS data);

/**************************************************** 
 * cyg_user_start
 * --------------------------------------------------
 *
 ****************************************************/
void cyg_user_start(void)
{

   static struct rtmon_cfg cfg;
   cfg.isr_mask    = RTMON_EVENT_ALL;
   cfg.dsr_mask    = RTMON_EVENT_ALL;
   cfg.event_mask  = RTMON_EVENT_SCHED_LOCK    | 
                     RTMON_EVENT_USER          | 
                     RTMON_EVENT_USER_U32      | 
                     RTMON_EVENT_THREAD_SWITCH |
                     RTMON_EVENT_THREAD_STATE  | 
                     RTMON_EVENT_CHECK_DSR_CNT;
   cfg.driver_mask = RTMON_DRIVER_AAI;
   memset( cfg.dsr_latency, 0, sizeof(cyg_uint32)*RTMON_IRQ_MAX );
   cfg.logfile     = NULL;


   diag_printf("starting\n");
   rtmon_start(&cfg);

   rtmon_set_user_event_label( 25, "false" );

   cyg_thread_create(20, thread1, (cyg_addrword_t) 0, "Thread1",
         (void *)stack[0], STACKSIZE, &thread[0], &thread_obj[0]);

   cyg_thread_create(21, thread2, (cyg_addrword_t) 0, "Thread2",
         (void *)stack[1], STACKSIZE, &thread[1], &thread_obj[1]);


   cyg_thread_create(19, thread3, (cyg_addrword_t) 0, "Thread3",
         (void *)stack[2], STACKSIZE, &thread[2], &thread_obj[2]);

   cyg_mutex_init(&mtx_inv_prio);

   cyg_thread_resume(thread[0]);
   cyg_thread_resume(thread[1]);
   cyg_thread_resume(thread[2]);
}




/**************************************************** 
 * thread1
 * --------------------------------------------------
 * Main test function
 ****************************************************/
#define WAIT_TIME    500
#define MAX_NB_SLEEP 20

int nb_sleep = 0 ;

void thread1(CYG_ADDRESS data)
{
   int i;

   rtmon_set_user_u32_label( 1,     "nb_sleep"  );
   rtmon_set_user_event_label( 1,   "while loop" );
   rtmon_set_user_event_label( 120, "thd1 locking mutex" );
   rtmon_set_user_event_label( 121, "thd2 locking mutex" );
   rtmon_set_user_event_label( 50,  "thd1 no real prio inv" );
   rtmon_set_user_event_label( 51,  "thd2 no real prio inv" );
   rtmon_set_user_event_label( 255, "locked" );

   //diag_printf("waiting for %ds ... ",WAIT_TIME/100 );
   //cyg_thread_delay(WAIT_TIME);
   //diag_printf("done \n " );

   /* First loop is supposed to test a real priority inversion:
    * means that a upper priority thread, modify a lower piority
    * thread */
   while (nb_sleep < MAX_NB_SLEEP) {
      cyg_thread_delay(1);
      rtmon_log_driver_start( 1 );
      rtmon_log_user_start( 120 );
	   cyg_mutex_lock(&mtx_inv_prio);
      rtmon_log_user_stop( 120 );
      /* locked section code */
      rtmon_log_user_start_timed( 255 );
      nb_sleep ++;
      rtmon_log_user_u32( 0, nb_sleep );
      rtmon_log_user_u32( 1, nb_sleep );
      rtmon_log_user_stop_timed( 255, 30 );
      /* end of locked section code */
	   cyg_mutex_unlock(&mtx_inv_prio);
      rtmon_log_driver_stop( 1 );
   };

   nb_sleep=0;
   /* Second loop is supposed to test an uneffective priority inversion:
    * means that a upper priority thread, do not modify a lower piority
    * thread */
   while (nb_sleep < MAX_NB_SLEEP) {
      cyg_thread_delay(1);
      rtmon_log_driver_start( 1 );
      rtmon_log_user_start( 120 );
	   cyg_mutex_lock(&mtx_inv_prio);
      rtmon_log_user_stop( 120 );
      /* locked section code */
      rtmon_log_user_start_timed( 50 );
      nb_sleep ++;
      rtmon_log_user_u32( 0, nb_sleep );
      rtmon_log_user_u32( 1, nb_sleep );
		i=0; do{i++;}while(i<BLOCKING_WAIT);
		if(nb_sleep == (4*4)){ i=0; do{i++;}while(i<BLOCKING_WAIT); };
      rtmon_log_user_stop_timed( 50, 50000 );
      /* end of locked section code */
	   cyg_mutex_unlock(&mtx_inv_prio);
      rtmon_log_driver_stop( 1 );
   };

   rtmon_flush("fin");
}

/**************************************************** 
 * thread2
 * --------------------------------------------------
 * Main test function
 ****************************************************/


void thread2(CYG_ADDRESS data)
{
	int i;

   while (nb_sleep < MAX_NB_SLEEP-1) 
   {
		if( (nb_sleep%4) == 0 )
		{
         rtmon_log_user_start( 121 );
			cyg_mutex_lock(&mtx_inv_prio);
         rtmon_log_user_stop( 121 );

         /* locked section code */
      	rtmon_log_user_start_timed( 255 );
  			i=0; do{i++;}while(i<BLOCKING_WAIT);
			if(nb_sleep == (4*4)){ i=0; do{i++;}while(i<BLOCKING_WAIT); };
      	rtmon_log_user_stop_timed( 255, 50000 );
         /* end of locked section code */
			cyg_mutex_unlock(&mtx_inv_prio);
        	rtmon_log_user_stop( 255 );
		}
	};

   while (nb_sleep < MAX_NB_SLEEP) 
   {
		if( (nb_sleep%4) == 0 )
		{
         rtmon_log_user_start( 51 );
			cyg_mutex_lock(&mtx_inv_prio);
         rtmon_log_user_stop( 51 );

         /* end of locked section code */
			cyg_mutex_unlock(&mtx_inv_prio);
        	rtmon_log_user_stop( 51 );
		}
	};

}


/**************************************************** 
 * thread3
 * --------------------------------------------------
 * Main test function
 ****************************************************/

void thread3(CYG_ADDRESS data)
{

   for(;;)
	{
      cyg_thread_delay(2);
	};
}


